//---------------------------------------------------------------------------

#include <vcl.h>
#include <dir.h>
#pragma hdrstop

#include "Hardware_.h"
#include "main_.h"
#include "keyboard_.h"
#include "kbstatus.h"
#include "zx81config.h"
#include "zx81.h"
#include "accdraw_.h"
#include "speed_.h"
#include "sound.h"
#include "rom.h"
#include "serialport.h"
extern "C" void sound_ay_init(void);

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
THardware *Hardware;
//---------------------------------------------------------------------------
__fastcall THardware::THardware(TComponent* Owner)
        : TForm(Owner)
{
        TIniFile *ini;

        Model->ItemIndex=1;
        RamPack->ItemIndex=4;
        Hres->ItemIndex=0;
        AY8910->ItemIndex=0;
        ColourBox->ItemIndex=0;
        SerialPort->ItemIndex=0;
        ChrGenBox->ItemIndex=0;

        ini = new TIniFile(ChangeFileExt(Application->ExeName, ".INI" ));
        LoadSettings(ini);
        Button1Click(NULL);
        delete ini;
}
//---------------------------------------------------------------------------

void __fastcall THardware::Button1Click(TObject *Sender)
{
        zx81.extfont=0;

        switch(Model->ItemIndex)
        {
        case 0:
                zx81.machine=MACHINEZX80;
                strcpy(zx81.ROM80, ROMBox->ROMFileName->Text.c_str());
                break;
        case 2:
                strcpy(zx81.ROM97LE, ROMBox->ROMFileName->Text.c_str());
                zx81.machine=MACHINEZX97LE;
                break;
        case 1:
                strcpy(zx81.ROM81, ROMBox->ROMFileName->Text.c_str());
                zx81.machine=MACHINEZX81;
                break;
        case 3:
                strcpy(zx81.ROM81, ROMBox->ROMFileName->Text.c_str());
                zx81.machine=MACHINEZX81;
                break;
        case 7:
                zx81.machine=MACHINEZX81;
                strcpy(zx81.ROMR470, ROMBox->ROMFileName->Text.c_str());
                break;
        case 8:
                strcpy(zx81.ROMTK85, ROMBox->ROMFileName->Text.c_str());
                zx81.machine=MACHINEZX81;
                break;
        case 4:
                zx81.machine=MACHINETS1500;
                strcpy(zx81.ROMTS1500, ROMBox->ROMFileName->Text.c_str());
                break;
        case 5:
        case 6:
                zx81.machine=MACHINELAMBDA;
                if (Model->ItemIndex==5)
                        strcpy(zx81.ROMLAMBDA, ROMBox->ROMFileName->Text.c_str());
                else
                        strcpy(zx81.ROMPC8300, ROMBox->ROMFileName->Text.c_str());
                zx81.extfont=1;
                break;
        case 9:
                zx81.machine=MACHINEACE;
                strcpy(zx81.ROMACE, ROMBox->ROMFileName->Text.c_str());
                break;
        }

        switch(ColourBox->ItemIndex)
        {
        case 1:
                if (zx81.machine==MACHINEACE)
                        zx81.colour=COLOURACE;
                else
                        zx81.colour=COLOURLAMBDA;
                break;
        default:
                zx81.colour=COLOURDISABLED;
                break;
        }

        Form1->InTapeManager->Enabled=true;
        Form1->InWaveLoader->Enabled=true;
        Form1->OutTapeManager->Enabled=true;
        Form1->OutWaveLoader->Enabled=true;

        zx81.shadowROM = ShadowROM->Checked;
        zx81.RAM816k = EnableRAM8k->Checked;
        zx81.protectROM = ProtectROM->Checked;
        zx81.chrgen = ChrGenBox->ItemIndex;

        if (NTSCMachine->Checked != zx81.NTSC)
        {
                zx81.NTSC = NTSCMachine->Checked;
                AccurateInit();
        }

        if (NTSCMachine->Checked) Form1->AnimTimer1->Interval=16;
        else Form1->AnimTimer1->Interval=20;


        zx81.RAMTOP = (1 << (RamPack->ItemIndex + 10)) +16383;
        if (RamPack->ItemIndex == 6) zx81.RAMTOP = 65535;
        if (zx81.machine==MACHINEACE && RamPack->ItemIndex==0) zx81.RAMTOP=16383;

        switch(Hres->ItemIndex)
        {
        case 1:
                zx81.truehires = HIRESWRX;
                break;
        case 2:
                zx81.truehires = HIRESG007;
                break;
        case 3:
                zx81.truehires = HIRESMEMOTECH;
                break;
        default:
        case 0:
                zx81.truehires = HIRESDISABLED;
                break;
        }

        switch(AY8910->ItemIndex)
        {
        case 1:
                zx81.aysound=1;
                zx81.aytype=AY_TYPE_QUICKSILVA;
                break;
        case 2:
                zx81.aysound=1;
                zx81.aytype=AY_TYPE_ZONX;
                break;
        case 3:
                zx81.aysound=1;
                zx81.aytype=AY_TYPE_FULLER;
                break;
        case 4:
                zx81.aysound=1;
                zx81.aytype=AY_TYPE_ACE;
                break;
        case 0:
        default:
                zx81.aysound=0;
                zx81.aytype=0;
        }
        ROMBox->OKClick(NULL);
        zx81.m1not = M1Not->Checked? 49152 : 32768;
        zx81.romtype=Model->ItemIndex;

        Form1->Zx97Config->Enabled=false;

        if (zx81.machine==MACHINEZX97LE)
        {
                zx81.RAMTOP=65535;
                zx81.m1not=49152;
                Form1->Zx97Config->Enabled=true;
        }

        if (SerialPort->ItemIndex==1)
        {
                zx81.ts2050=1;
                Form1->TS20501->Enabled=true;
        }
        else
        {
                zx81.ts2050=0;
                Form1->TS20501->Enabled=false;
        }

        PCKbInit();
        zx81_init();
        sound_ay_init();
        Keyboard->KbChange();

        if (Sender)
        {
                Close();
        }
}
//---------------------------------------------------------------------------
void __fastcall THardware::EnableRAM8kClick(TObject *Sender)
{
        if (EnableRAM8k->Checked) ShadowROM->Checked=false;
}
//---------------------------------------------------------------------------

void __fastcall THardware::ModelChange(TObject *Sender)
{
        int temp;

        RamPack->Enabled=true;
        ShadowROM->Enabled=true;
        EnableRAM8k->Enabled=true;
        Hres->Enabled=true;
        ColourBox->Enabled=true;
        ShadowROM->Enabled=true;
        EnableRAM8k->Enabled=true;
        M1Not->Enabled=true;
        temp=ColourBox->ItemIndex;
        ColourBox->Items->Strings[1]="Lambda";
        ColourBox->ItemIndex=temp;
        ChrGenBox->Enabled=true;

        switch(Model->ItemIndex)
        {
        case 0:
                ROMBox->ROMFileName->Text = zx81.ROM80;
                break;
        case 1:
                ROMBox->ROMFileName->Text = zx81.ROM81;
                NTSCMachine->Checked=false;
                break;
        case 2:
                RamPack->ItemIndex=0;
                RamPack->Enabled=false;
                ShadowROM->Checked=false;
                ShadowROM->Enabled=false;
                EnableRAM8k->Checked=true;
                EnableRAM8k->Enabled=false;
                Hres->ItemIndex=1;
                Hres->Enabled=false;
                M1Not->Checked=true;
                M1Not->Enabled=false;
                ChrGenBox->ItemIndex=3;
                ChrGenBox->Enabled=false;

                ROMBox->ROMFileName->Text = zx81.ROM97LE;
                break;
        case 3:
                if (RamPack->ItemIndex<1) RamPack->ItemIndex=1;
                ROMBox->ROMFileName->Text = zx81.ROM81;
                NTSCMachine->Checked=true;
                break;

        case 8:
                ROMBox->ROMFileName->Text = zx81.ROMTK85;
                if (RamPack->ItemIndex<4) RamPack->ItemIndex=4;
                NTSCMachine->Checked=true;
                break;
        case 4:
                ROMBox->ROMFileName->Text = zx81.ROMTS1500;
                NTSCMachine->Checked = true;
                if (RamPack->ItemIndex<4) RamPack->ItemIndex=4;
                break;
        case 5:
        case 6:
                if (RamPack->ItemIndex<1) RamPack->ItemIndex=1;
                if (Model->ItemIndex==5) ROMBox->ROMFileName->Text = zx81.ROMLAMBDA;
                else ROMBox->ROMFileName->Text = zx81.ROMPC8300;
                NTSCMachine->Checked=true;
                break;
        case 7:
                ROMBox->ROMFileName->Text = zx81.ROMR470;
                NTSCMachine->Checked = true;
                ShadowROM->Checked=true;
                EnableRAM8k->Checked=false;
                if (RamPack->ItemIndex<4) RamPack->ItemIndex=4;
                break;
        case 9:
                ROMBox->ROMFileName->Text = zx81.ROMACE;
                Hres->ItemIndex=0;
                temp=ColourBox->ItemIndex;
                ColourBox->Items->Strings[1]="ETI Colour";
                ColourBox->ItemIndex=temp;
                ShadowROM->Checked=false;
                EnableRAM8k->Checked=false;
                Hres->Enabled=false;
                ShadowROM->Enabled=false;
                EnableRAM8k->Enabled=false;
                M1Not->Checked=false;
                M1Not->Enabled=false;
                break;
        }
        FormShow(NULL);
}
//---------------------------------------------------------------------------

void __fastcall THardware::ShadowROMClick(TObject *Sender)
{
        if (ShadowROM->Checked) EnableRAM8k->Checked=false;
}
//---------------------------------------------------------------------------

void THardware::LoadSettings(TIniFile *ini)
{
        Top = ini->ReadInteger("HW","Top",Top);
        Left = ini->ReadInteger("HW","Left",Left);
        //Height = ini->ReadInteger("HW","Height",Height);
        //Width = ini->ReadInteger("HW","Width",Width);

        Model->ItemIndex = ini->ReadInteger("HW","Model", Model->ItemIndex);
        ModelChange(NULL);
        RamPack->ItemIndex = ini->ReadInteger("HW","RAM", RamPack->ItemIndex);
        //ROMFileName->Text = ini->ReadString("HW","ROM", ROMFileName->Text);
        Hres->ItemIndex = ini->ReadInteger("HW","HRES", Hres->ItemIndex);
        AY8910->ItemIndex = ini->ReadInteger("HW","AY8910", AY8910->ItemIndex);
        ColourBox->ItemIndex = ini->ReadInteger("HW","COLOUR", ColourBox->ItemIndex);
        SerialPort->ItemIndex = ini->ReadInteger("HW","SERIAL", SerialPort->ItemIndex);
        ChrGenBox->ItemIndex = ini->ReadInteger("HW","CHRGEN", ChrGenBox->ItemIndex);

        ShadowROM->Checked = ini->ReadBool("HW","ShadowROM", ShadowROM->Checked);
        EnableRAM8k->Checked = ini->ReadBool("HW","EnableRAM8k", EnableRAM8k->Checked);
        ProtectROM->Checked = ini->ReadBool("HW","ProtectROM", ProtectROM->Checked);
        M1Not->Checked = ini->ReadBool("HW","M1Not", M1Not->Checked);

        NTSCMachine->Checked = ini->ReadBool("HW","NTSC", NTSCMachine->Checked);
        SerialPortChange(NULL);
        Button1Click(NULL);

}

void THardware::SaveSettings(TIniFile *ini)
{
        ini->WriteInteger("HW","Top",Top);
        ini->WriteInteger("HW","Left",Left);
        //ini->WriteInteger("HW","Height",Height);
        //ini->WriteInteger("HW","Width",Width);

        ini->WriteInteger("HW","Model", Model->ItemIndex);
        ini->WriteInteger("HW","RAM", RamPack->ItemIndex);
        ini->WriteInteger("HW","HRES", Hres->ItemIndex);
        ini->WriteInteger("HW","AY8910", AY8910->ItemIndex);
        ini->WriteInteger("HW","COLOUR", ColourBox->ItemIndex);
        ini->WriteInteger("HW","SERIAL", SerialPort->ItemIndex);
        ini->WriteInteger("HW","CHRGEN", ChrGenBox->ItemIndex);
        //ini->WriteString("HW","ROM", ROMFileName->Text);

        ini->WriteBool("HW","ShadowROM", ShadowROM->Checked);
        ini->WriteBool("HW","EnableRAM8k", EnableRAM8k->Checked);
        ini->WriteBool("HW","ProtectROM", ProtectROM->Checked);
        ini->WriteBool("HW","M1Not", M1Not->Checked);
        ini->WriteBool("HW","NTSC", NTSCMachine->Checked);
}

void __fastcall THardware::HresChange(TObject *Sender)
{
        if (Hres->ItemIndex==2)
        {
                ShadowROM->Enabled=false;
                ShadowROM->Checked=false;
                EnableRAM8k->Enabled=false;
                EnableRAM8k->Checked=true;
        }
        else
        {
                ShadowROM->Enabled=true;
                EnableRAM8k->Enabled=true;
        }
}
//---------------------------------------------------------------------------

void __fastcall THardware::ColourBoxChange(TObject *Sender)
{
        if (ColourBox->ItemIndex==1) ShadowROM->Checked=false;
}
//---------------------------------------------------------------------------

void __fastcall THardware::RomFileClick(TObject *Sender)
{
        ROMBox->ShowModal();
        RomFile->Items->Clear();
        RomFile->Items->Add(ROMBox->ROMFileName->Text);
}
//---------------------------------------------------------------------------

void __fastcall THardware::FormShow(TObject *Sender)
{
        RomFile->Items->Clear();
        RomFile->Items->Add(ROMBox->ROMFileName->Text);
}
//---------------------------------------------------------------------------


void __fastcall THardware::SerialPortChange(TObject *Sender)
{
        SerialConfigBtn->Enabled= (SerialPort->ItemIndex != 0) ? true : false;
}
//---------------------------------------------------------------------------

void __fastcall THardware::SerialConfigBtnClick(TObject *Sender)
{
        SerialConfig->Show();
}
//---------------------------------------------------------------------------



