//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>

#pragma hdrstop

#include "TapeMan_.h"
#include "main_.h"
#include "utils.h"
#include "wavload_.h"
#include "zx81config.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

extern "C" int load_tape_buffer(int ptr);
extern "C" int set_PC(int addr);

TTape *Tape;
//---------------------------------------------------------------------------
void __fastcall TTape::WMGetMinMaxInfo(TWMGetMinMaxInfo &Message)
{
        Message.MinMaxInfo->ptMinTrackSize.x = 285;
        Message.MinMaxInfo->ptMinTrackSize.y = 161;
        Message.MinMaxInfo->ptMaxTrackSize.x = 285;
        Message.MinMaxInfo->ptMaxTrackSize.y = 768;
}
//---------------------------------------------------------------------------
__fastcall TTape::TTape(TComponent* Owner)
        : TForm(Owner)
{
        TIniFile *ini;
        int i;

        DragAcceptFiles(Tape->Handle, true);

        //StringGrid->Rows[0]->CommaText=",Name,Length";
        CurrentPosition = 0;
        Changed=false;

        for(i=0; i<MAXFILES; i++) File[i]=NULL;
        //StringGrid->Col[0]->Align= alRight;
        RecalcCurPos();

        ini = new TIniFile(ChangeFileExt(Application->ExeName, ".INI" ));
        LoadSettings(ini);
        delete ini;

}
//---------------------------------------------------------------------------
int TTape::LoadBlock(AnsiString filename)
{
        FILE *fptr;
        char zxfilename[128];

        fptr=fopen(filename.c_str(), "rb");
        if (!fptr) return(false);

        fseek(fptr, 0, SEEK_END);
        NewFileLen=ftell(fptr);
        fseek(fptr, 0 , SEEK_SET);
        if (NewFileLen<32)
        {
                fclose(fptr);
                return(false);
        }

        ConvertASCIIZX81(RemoveExt(RemovePath(filename)), zxfilename);



        NewFile=(BYTE *) malloc(NewFileLen+ZX81Strlen(zxfilename));
        memcpy(NewFile, zxfilename, ZX81Strlen(zxfilename));

        fread(NewFile+ZX81Strlen(zxfilename), NewFileLen, 1, fptr);
        fclose(fptr);
        return(true);
}

//---------------------------------------------------------------------------
int TTape::SaveBlock(AnsiString filename,FILE *fp, int Block)
{
        FILE *fptr;

        if (fp) fptr=fp;
        else
        {
                fptr=fopen(filename.c_str(), "wb");
                if (!fptr) return(false);
        }

        fwrite(File[Block] + ZX81Strlen(File[Block]), 1,
                  (StringGrid->Cells[2][Block].ToInt()) - ZX81Strlen(File[Block]), fptr);
        fflush(fptr);
        fclose(fptr);
        return(true);
}

//---------------------------------------------------------------------------
void __fastcall TTape::StringGridDblClick(TObject *Sender)
{
        CurrentPosition = StringGrid->Row;
        RecalcCurPos();
        //StringGrid->Rows[CurrentPosition]->Strings[0]=">";

}

//---------------------------------------------------------------------------

void TTape::RecalcCurPos()
{
        int i,j;
        bool changed;

        do
        {
                changed=false;
                for(i=0;i<(StringGrid->RowCount);i++)
                {
                        if ((StringGrid->Cells[1][i] == "<Silence>") && (StringGrid->Cells[1][i+1] == "<Silence>"))
                        {
                                int CurrentSilence;
                                AnsiString time;

                                CurrentSilence = atoi((StringGrid->Cells[2][i]).c_str());
                                CurrentSilence += atoi((StringGrid->Cells[2][i+1]).c_str());

                                time = CurrentSilence;
                                time += " ms";
                                StringGrid->Cells[2][i] = time;
                                StringGrid->Cells[1][i+1]= "xxx";
                                StringGrid->Cells[2][i+1] = "xxx";

                                if (CurrentPosition>=i) CurrentPosition--;
                                for(j=i+1; j<StringGrid->RowCount; j++)
                                {
                                        StringGrid->Rows[j]=StringGrid->Rows[j+1];
                                        File[j]=File[j+1];
                                }

                                StringGrid->RowCount--;

                                changed=true;
                        }
                }
        } while(changed);


        if (CurrentPosition<0) CurrentPosition=0;
        if (CurrentPosition > StringGrid->RowCount-1)
                CurrentPosition = StringGrid->RowCount-1;

        for(i=0; i<StringGrid->RowCount; i++)
        {
                StringGrid->Cells[0][i]="";
                if (i==CurrentPosition)
                        StringGrid->Cells[0][i]="--------->";
        }

        StringGrid->Cells[1][StringGrid->RowCount-1]=" ";
        StringGrid->Cells[2][StringGrid->RowCount-1]=" ";
}
//---------------------------------------------------------------------------

int TTape::SaveTapeFile(AnsiString FileName)
{
        char header[] = HEADER_ID;
        char fname[32], flen[16];

        FILE *fptr;
        int i;
        int error=0;

        fptr=fopen(FileName.c_str(), "wb");
        if (!fptr) return(false);

        error += fwrite(header, strlen(header), 1, fptr);

        for(i=0; i < (StringGrid->RowCount - 1); i++)
        {
                memset(fname, 0, 32);
                memset(flen, 0, 16);

                strncpy(fname, StringGrid->Cells[1][i].c_str(), 31);
                strncpy(flen, StringGrid->Cells[2][i].c_str(), 15);

                error += fwrite(fname, 32, 1, fptr);
                error += fwrite(flen, 16, 1, fptr);

                if (StringGrid->Cells[1][i] != "<Silence>")
                        error += fwrite(File[i], atoi(flen), 1, fptr);
        }

        fclose(fptr);

        if (error) return(false);
        return(true);
}
//---------------------------------------------------------------------------

int TTape::LoadTapeFile(AnsiString FileName)
{
        AnsiString Ext;

        char header[5];
        char fname[32], flen[16];
        char buffer[65536], zxnamebuf[32];

        FILE *fptr;
        int i;
        int length, zxnamelen;

        CloseTape();

        if (FileName.Length()<3) return(false);

        Ext=GetExt(FileName);

        if (Ext==".P" || Ext==".O")
        {
                if (!LoadBlock(FileName)) return(false);
                Tape->FileName = FileName;
                BlockName="";
                InsertBlock1Click(NULL);
                RecalcCurPos();
                Changed=false;
                Caption="Tape Manager - " + FileNameGetFname(FileName)+FileNameGetExt(FileName);
                zx81.autoload=1;
                return(true);
        }

        i=-1;

        fptr=fopen(FileName.c_str(), "rb");
        if (!fptr) return(false);

        fread(header, 4, 1, fptr);
        if (strncmp(header, HEADER_ID,4)) return(false);

        do
        {
                memset(fname, 0, 32);
                memset(flen, 0, 16);
                fread(fname, 32, 1, fptr);
                fread (flen, 16, 1, fptr);

                length = atoi(flen);

                if ( (strlen(fname)>29) || (length < 20) || (length > 65535) )
                        break;
                i++;
                if (strcmp(fname,"<Silence>"))
                {
                        fread(buffer, length, 1, fptr);

                        if ( (*buffer==0x00) || (*buffer==-1) || (*buffer==1) ) // If buffer doesn't include the filename, add one
                        {
                                ConvertASCIIZX81(fname, zxnamebuf);
                                zxnamelen = ZX81Strlen(zxnamebuf);
                        }
                        else    zxnamelen = 0;


                        StringGrid->RowCount ++;

                        StringGrid->Cells[1][i] = fname;
                        StringGrid->Cells[2][i] = length+zxnamelen;

                        File[i] = (BYTE *) malloc(length+zxnamelen);

                        memcpy(File[i], zxnamebuf, zxnamelen);
                        memcpy(File[i]+zxnamelen, buffer, length);
                }
                else
                {
                        StringGrid->RowCount ++;

                        StringGrid->Cells[1][i] = fname;
                        StringGrid->Cells[2][i] = flen;
                        File[i] = NULL;
                }



                //fread(File[i], length, 1, fptr);
        } while(!feof(fptr));

        fclose(fptr);
        RecalcCurPos();
        Changed=false;
        Caption="Tape Manager - " + FileNameGetFname(FileName)+FileNameGetExt(FileName);
        zx81.autoload=1;
        return(true);
}

//---------------------------------------------------------------------------

void TTape::Rewind(void)
{
        CurrentPosition=0;
        RecalcCurPos();
}

void TTape::NextFile(void)
{
        CurrentPosition++;
        if (CurrentPosition == StringGrid->RowCount)
                CurrentPosition--;
        RecalcCurPos();
}

BYTE *TTape::GetCurrentFile(void)
{
        BYTE *p;

//        while(StringGrid->Cells[1][CurrentPosition] == "<Silence>") NextFile();

        p=File[CurrentPosition];
        if (*p!=0 && /* *p!=-1 &&*/ *p!=255 && *p!=1) p += ZX81Strlen(p);
        return(p);
}

int TTape::GetCurrentFileLen(void)
{
        if (CurrentPosition == (StringGrid->RowCount-1)) return(0);
        return( (StringGrid->Cells[2][CurrentPosition].ToInt())
                        - ZX81Strlen(File[CurrentPosition]) );
}

bool TTape::FindFile(BYTE *zx81Name)
{
        AnsiString Name;
        int p;

        Name="";
        if (zx81Name) Name = ConvertZX81ASCII(zx81Name);

        if (StringGrid->RowCount==1) return(false);

        if (Name=="")
        {
                while(StringGrid->Cells[1][CurrentPosition] == "<Silence>") CurrentPosition++;
                if (CurrentPosition == StringGrid->RowCount-1 && AutoRewind->Checked) CurrentPosition=0;
                while(StringGrid->Cells[1][CurrentPosition] == "<Silence>") CurrentPosition++;

                if (CurrentPosition != StringGrid->RowCount-1)
                        return(true);

                //if (AutoRewind->Checked)
                //{
                //        CurrentPosition=0;
                //        return(true);
                //}

                return(false);
        }

        p=CurrentPosition;

        do
        {
                if ( (p >= (StringGrid->RowCount-1)) && AutoRewind->Checked) p=0;

                if (StringGrid->Cells[1][p] == Name)
                {
                        CurrentPosition=p;
                        RecalcCurPos();
                        return(true);
                }

                p++;

        } while ( (p!=CurrentPosition && AutoRewind->Checked)
                     || (p<(StringGrid->RowCount-1) && !AutoRewind->Checked) );

        CurrentPosition = p;
        RecalcCurPos();
        return(false);
}

void TTape::AddBlock(BYTE *zx81Name, BYTE *ptr, int Len)
{
        int NewRow,i;
        AnsiString Name;

        Changed = true;

        NewRow = CurrentPosition;
        StringGrid->RowCount++;

        if (NewRow != (StringGrid->RowCount - 2))
                for (i= StringGrid->RowCount-2; i>=NewRow; i--)
                {
                        StringGrid->Rows[i+1] = StringGrid->Rows[i];
                        File[i+1] = File[i];
                }

        StringGrid->Row = NewRow+1;

        CurrentPosition++;

        if (zx81Name && ptr)
        {
                Name = ConvertZX81ASCII(zx81Name);
                StringGrid->Cells[1][NewRow]=Name;
                StringGrid->Cells[2][NewRow]=Len + ZX81Strlen(zx81Name);

                File[NewRow]=(BYTE *) malloc(Len+ZX81Strlen(zx81Name));
                memcpy(File[NewRow], zx81Name, ZX81Strlen(zx81Name));
                memcpy(File[NewRow]+ZX81Strlen(zx81Name), ptr, Len);
        }
        else
        {
                AnsiString time;

                time = (Len/3250);
                time = time + " ms";

                StringGrid->Cells[1][NewRow]="<Silence>";
                StringGrid->Cells[2][NewRow]=time;
                File[NewRow]=NULL;
        }
        Changed=true;
        RecalcCurPos();
}

void TTape::NewTape(void)
{
        CloseTape();
}

void TTape::CloseTape(void)
{
        int i;

        if (Changed)
        {
                int btn;

                btn=Application->MessageBox("The current tape file has not been saved.\nWould you like to save it before continuing?",
                                 "Save?", MB_YESNO);

                if (btn==IDYES)SaveTape1Click(NULL);
        }

        for(i=0; i<MAXFILES; i++)
                if (File[i])
                {
                        free(File[i]);
                        File[i]=0;
                }

        //StringGrid->Rows[0]->Delete();

        CurrentPosition=0;
        StringGrid->RowCount=1;
        StringGrid->Rows[0]->CommaText=",,";
        RecalcCurPos();
        Caption="Tape Manager";
        Changed=false;
}

void __fastcall TTape::FormResize(TObject *Sender)
{
        Width=289;
}
//---------------------------------------------------------------------------

void __fastcall TTape::FormClose(TObject *Sender, TCloseAction &Action)
{
        Form1->Tape1->Checked=false;
}
//---------------------------------------------------------------------------

bool TTape::IsTape(AnsiString FileName)
{
        AnsiString Ext;

        Ext = GetExt(FileName);

        if (Ext==".T81" || Ext==".P" || Ext==".81" || Ext==".O") return(true);
        return(false);
}
//---------------------------------------------------------------------------

void TTape::LoadSettings(TIniFile *ini)
{
        Top = ini->ReadInteger("TAPE","Top",Top);
        Left = ini->ReadInteger("TAPE","Left",Left);
        Height = ini->ReadInteger("TAPE","Height",Height);
        Width = ini->ReadInteger("TAPE","Width",Width);

        AutoRewind->Checked = ini->ReadBool("TAPE","AutoRew", AutoRewind->Checked);
        AutoLoad->Checked = ini->ReadBool("TAPE","AutoLoad", AutoLoad->Checked);

        LoadTapeDialog->FileName = ini->ReadString("TAPE","LoadTapeFile", LoadTapeDialog->FileName);
        LoadTapeDialog->InitialDir = ini->ReadString("TAPE","LoadTapeDir", LoadTapeDialog->InitialDir);
        LoadTapeDialog->FilterIndex = ini->ReadInteger("TAPE","LoadTapeType", LoadTapeDialog->FilterIndex);

        InsertBlockDialog->FileName = ini->ReadString("TAPE","InsertBlockFile", InsertBlockDialog->FileName);
        InsertBlockDialog->InitialDir = ini->ReadString("TAPE","InsertBlockDir", InsertBlockDialog->InitialDir);
        InsertBlockDialog->FilterIndex = ini->ReadInteger("TAPE","InsertBlockType", InsertBlockDialog->FilterIndex);

        SaveTapeDialog->DefaultExt = ini->ReadString("TAPE","SaveTapeExt", SaveTapeDialog->DefaultExt);
        SaveTapeDialog->FileName = ini->ReadString("TAPE","SaveTapeFile", SaveTapeDialog->FileName);
        SaveTapeDialog->InitialDir = ini->ReadString("TAPE","SaveTapeDir", SaveTapeDialog->InitialDir);
        SaveTapeDialog->FilterIndex = ini->ReadInteger("TAPE","SaveTapeType", SaveTapeDialog->FilterIndex);

        ExtractBlockDialog->DefaultExt = ini->ReadString("TAPE","ExtractBlockExt", ExtractBlockDialog->DefaultExt);
        ExtractBlockDialog->FileName = ini->ReadString("TAPE","ExtractBlockFile", ExtractBlockDialog->FileName);
        ExtractBlockDialog->InitialDir = ini->ReadString("TAPE","ExtractBlockDir", ExtractBlockDialog->InitialDir);
        ExtractBlockDialog->FilterIndex = ini->ReadInteger("TAPE","ExtractBlockType", ExtractBlockDialog->FilterIndex);

        if (Form1->Tape1->Checked) Show();
}

void TTape::SaveSettings(TIniFile *ini)
{
        ini->WriteInteger("TAPE","Top",Top);
        ini->WriteInteger("TAPE","Left",Left);
        ini->WriteInteger("TAPE","Height",Height);
        ini->WriteInteger("TAPE","Width",Width);

        ini->WriteBool("TAPE","AutoRew", AutoRewind->Checked);
        ini->WriteBool("TAPE","AutoLoad", AutoLoad->Checked);

        ini->WriteString("TAPE","LoadTapeFile", LoadTapeDialog->FileName);
        ini->WriteString("TAPE","LoadTapeDir", LoadTapeDialog->InitialDir);
        ini->WriteInteger("TAPE","LoadTapeType", LoadTapeDialog->FilterIndex);

        ini->WriteString("TAPE","InsertBlockFile", InsertBlockDialog->FileName);
        ini->WriteString("TAPE","InsertBlockDir", InsertBlockDialog->InitialDir);
        ini->WriteInteger("TAPE","InsertBlockType", InsertBlockDialog->FilterIndex);

        ini->WriteString("TAPE","SaveTapeExt", SaveTapeDialog->DefaultExt);
        ini->WriteString("TAPE","SaveTapeFile", SaveTapeDialog->FileName);
        ini->WriteString("TAPE","SaveTapeDir", SaveTapeDialog->InitialDir);
        ini->WriteInteger("TAPE","SaveTapeType", SaveTapeDialog->FilterIndex);

        ini->WriteString("TAPE","ExtractBlockExt", ExtractBlockDialog->DefaultExt);
        ini->WriteString("TAPE","ExtractBlockFile", ExtractBlockDialog->FileName);
        ini->WriteString("TAPE","ExtractBlockDir", ExtractBlockDialog->InitialDir);
        ini->WriteInteger("TAPE","ExtractBlockType", ExtractBlockDialog->FilterIndex);
}

//---------------------------------------------------------------------------



void __fastcall TTape::NewTape1Click(TObject *Sender)
{
        NewTape();        
}
//---------------------------------------------------------------------------

void __fastcall TTape::OpenTape1Click(TObject *Sender)
{
        CloseTape();
        if (!LoadTapeDialog->Execute()) return;

        if (LoadTapeFile(LoadTapeDialog->FileName))
        {
                FileName = LoadTapeDialog->FileName;
                Changed=false;
                if (AutoLoad->Checked)
                {
                        load_tape_buffer(65536);
                        set_PC(0x0207);
                }
        }        
}
//---------------------------------------------------------------------------

void __fastcall TTape::SaveTape1Click(TObject *Sender)
{
        AnsiString Filter = "EightyOne Tape File (T81)|*.T81";

        if (FileName != "") SaveTapeDialog->FileName = RemoveExt(FileName);
        else SaveTapeDialog->FileName = RemoveExt(SaveTapeDialog->FileName);

        if (StringGrid->RowCount==2) Filter += "|.P ZX81|*.P";
        if (StringGrid->RowCount==2) Filter += "|.O ZX80|*.O";

        SaveTapeDialog->Filter = Filter;
        SaveTapeDialog->DefaultExt = "T81";
        SaveTapeDialog->FilterIndex=1;


        if (FileName.Length())
                if (toupper(FileName[FileName.Length()]) == 'P')
                {
                        SaveTapeDialog->DefaultExt = "p";
                        SaveTapeDialog->FilterIndex= 2;
                }

        if (!SaveTapeDialog->Execute()) return;

        if (GetExt(SaveTapeDialog->FileName) == ".P" || GetExt(SaveTapeDialog->FileName) == ".O")
        {
                SaveBlock(SaveTapeDialog->FileName,0, 0);
                FileName = SaveTapeDialog->FileName;
        }
        else    SaveTapeFile(SaveTapeDialog->FileName);
        Changed=false;        
}
//---------------------------------------------------------------------------

void __fastcall TTape::TapetoWave1Click(TObject *Sender)
{
        int i;

        for(i=0; i<(StringGrid->RowCount-1);i++)
                WavLoad->EncodeBlock(File[i], atoi((StringGrid->Cells[2][i]).c_str()));        
}
//---------------------------------------------------------------------------

void __fastcall TTape::BlocktoWave1Click(TObject *Sender)
{
        WavLoad->EncodeBlock(File[StringGrid->Row], atoi((StringGrid->Cells[2][StringGrid->Row]).c_str()));        
}
//---------------------------------------------------------------------------

void __fastcall TTape::InsertBlock1Click(TObject *Sender)
{
        AnsiString Name;
        int NewRow,i;

        if (Sender)
        {
                if (!InsertBlockDialog->Execute()) return;
                Name = InsertBlockDialog->FileName;
                if (!LoadBlock(Name)) return;
        }
//        else
//        {
//                if (BlockName=="") Name = FileName;
//                else Name=BlockName;
//        }

        Changed = true;

        NewRow = StringGrid->Row;
        StringGrid->RowCount++;

        if (StringGrid->Row != (StringGrid->RowCount - 2))
                for (i= StringGrid->RowCount-2; i>=NewRow; i--)
                {
                        StringGrid->Rows[i+1] = StringGrid->Rows[i];
                        File[i+1] = File[i];
                }

        StringGrid->Row = NewRow+1;

        if (NewRow < CurrentPosition) CurrentPosition++;
        RecalcCurPos();

//        Name = ExtractFileName(Name);

//        for(i=1; i<= Name.Length(); i++) Name[i] = toupper(Name[i]);

//        i=Name.AnsiPos(".P"); if (i) Name = Name.SubString(1, i-1);
//        i=Name.AnsiPos(".81"); if (i) Name = Name.SubString(1, i-2);

        StringGrid->Rows[NewRow]->Strings[1] = ConvertZX81ASCII(NewFile);
        StringGrid->Rows[NewRow]->Strings[2] = NewFileLen + ZX81Strlen(NewFile);
        File[NewRow]=NewFile;


}
//---------------------------------------------------------------------------


void __fastcall TTape::ExtractBlock1Click(TObject *Sender)
{
        char type;

        type =  *(File[StringGrid->Row]+ZX81Strlen(File[StringGrid->Row]));

        if (StringGrid->Row == (StringGrid->RowCount-1)) return;
        if (StringGrid->Cells[1][StringGrid->Row] == "<Silence>") return;

        ExtractBlockDialog->FileName = StringGrid->Cells[1][StringGrid->Row];

        if (type == -1)
        {
                ExtractBlockDialog->FileName += ".o";
                ExtractBlockDialog->Filter = ".o|*.o";
        }
        else
        {
                ExtractBlockDialog->FileName += ".p";
                ExtractBlockDialog->Filter = ".p|*.p";
        }

        if (!ExtractBlockDialog->Execute()) return;

        SaveBlock(ExtractBlockDialog->FileName, 0, StringGrid->Row);
}
//---------------------------------------------------------------------------

void __fastcall TTape::MoveBlockUp1Click(TObject *Sender)
{
        AnsiString row;
        BYTE *file;

        if (StringGrid->Row==0 || StringGrid->Row==StringGrid->RowCount-1) return;

        file=File[StringGrid->Row];
        row=StringGrid->Rows[StringGrid->Row]->CommaText;

        File[StringGrid->Row] = File[StringGrid->Row-1];
        StringGrid->Rows[StringGrid->Row]->CommaText =
                StringGrid->Rows[StringGrid->Row-1]->CommaText;

        StringGrid->Row -= 1;

        File[StringGrid->Row] = file;
        StringGrid->Rows[StringGrid->Row]->CommaText = row;

        Changed=true;
}
//---------------------------------------------------------------------------

void __fastcall TTape::MoveBlockDown1Click(TObject *Sender)
{
        AnsiString row;
        BYTE *file;

        if (StringGrid->Row >= StringGrid->RowCount-2) return;

        file=File[StringGrid->Row];
        row=StringGrid->Rows[StringGrid->Row]->CommaText;

        File[StringGrid->Row] = File[StringGrid->Row+1];
        StringGrid->Rows[StringGrid->Row]->CommaText =
                StringGrid->Rows[StringGrid->Row+1]->CommaText;

        StringGrid->Row += 1;

        File[StringGrid->Row] = file;
        StringGrid->Rows[StringGrid->Row]->CommaText = row;

        Changed=true;
}
//---------------------------------------------------------------------------

void __fastcall TTape::DeleteBlock1Click(TObject *Sender)
{
        int i;

        if (StringGrid->Row >= (StringGrid->RowCount-1)) return;

        if (StringGrid->Row < CurrentPosition) CurrentPosition--;
        if (File[StringGrid->Row]) free(File[StringGrid->Row]);

        for(i=StringGrid->Row; i<StringGrid->RowCount; i++)
        {
                StringGrid->Rows[i]=StringGrid->Rows[i+1];
                File[i]=File[i+1];
        }

        StringGrid->RowCount--;
        RecalcCurPos();
        Changed=true;
}
//---------------------------------------------------------------------------



void __fastcall TTape::AutoRewindClick(TObject *Sender)
{
        AutoRewind->Checked = !AutoRewind->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TTape::AutoLoadClick(TObject *Sender)
{
        AutoLoad->Checked = !AutoLoad->Checked;
}
//---------------------------------------------------------------------------

