//---------------------------------------------------------------------------

#include <vcl.h>
#include <io.h>
#pragma hdrstop

#include "zx81.h"
#include "z80thread.h"
#include "kbstatus.h"
#include "snap.h"
#include "zx81config.h"
#include "accurate.h"
#include "sound.h"

#include "main_.h"
#include "accdraw_.h"
#include "About_.h"
#include "keyboard_.h"
#include "hardware_.h"
#include "speed_.h"
#include "kb_.h"
#include "tapeman_.h"
#include "zxprinter.h"
#include "wavload_.h"
#include "debug.h"
#include "utils.h"
#include "artifacts_.h"
#include "SoundOP.h"
#include "rom.h"
#include "memsave_.h"
#include "zx97config.h"
#include "SerialPort.cpp"
#include "tzxman.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "dbits"
#pragma link "AnimTimer"
#pragma resource "*.dfm"

TForm1 *Form1;
//Tz80 *z80thread;

extern "C" void z80_reset();
extern char *CommandLine;
extern "C" int tsmax;
extern "C" void ramwobble(int now);

//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
        FullScreen=false;
        startup=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
        TIniFile *ini;
        int i;

        DragAcceptFiles(Form1->Handle, true);
        Application->OnMessage = AppMessage;

        sound_init();
        load_config();

        strcpy(zx81.cwd, (FileNameGetPath(Application->ExeName)).c_str());

        AccurateInit();

        PCKbInit();

        Application->OnDeactivate=FormDeactivate;

        ini = new TIniFile(ChangeFileExt(Application->ExeName, ".ini" ));
        LoadSettings(ini);
        delete ini;

        AnimTimer1->Interval=20;
        AnimTimer1->Enabled=true;

        for(i=0;i<256;i++)
        {
                DImage1->Palette[i] = DFlipSurface->Palette[i];
                DImage2->Palette[i] = DFlipSurface->Palette[i];
                DImage3->Palette[i] = DFlipSurface->Palette[i];
        }

        if (!access("eightyone.chm",0)) HelpTopics2->Enabled=true;
        else HelpTopics2->Enabled=false;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormResize(TObject *Sender)
{
        extern void RecalcWinSize(void);

        DFlipSurface->Width=ClientWidth;
        DFlipSurface->Height=ClientHeight;

        N1001->Checked=false;
        N2001->Checked=false;
        N4001->Checked=false;
        UserDefined1->Checked=false;

        if (ClientWidth==BaseWidth && ClientHeight==BaseHeight)
                N1001->Checked=true;
        else if (ClientWidth==(BaseWidth*2) && ClientHeight==(BaseHeight*2))
                N2001->Checked=true;
        else if (ClientWidth==(BaseWidth*4) && ClientHeight==(BaseHeight*4))
                N4001->Checked=true;
        else UserDefined1->Checked=true;

        RecalcWinSize();
}

//---------------------------------------------------------------------------

void __fastcall TForm1::FormKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
        PCKeyDown(Key);
        if (zx81.wobble) ramwobble(false);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormKeyUp(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
        PCKeyUp(Key);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::AboutEightyOne1Click(TObject *Sender)
{
        PCAllKeysUp();
        About->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Exit1Click(TObject *Sender)
{
        Close();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::KeyboardMap1Click(TObject *Sender)
{
        if (!KeyboardMap1->Checked)
        {
                PCAllKeysUp();
                Keyboard->Show();
                KeyboardMap1->Checked=true;
        }
        else
        {
                Keyboard->Close();
                KeyboardMap1->Checked=false;
        }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::N1001Click(TObject *Sender)
{
        N1001->Checked=true;
        N2001->Checked=false;
        N4001->Checked=false;
        UserDefined1->Checked=false;

        ClientWidth=BaseWidth;
        ClientHeight=BaseHeight;
}
//--
//-----------------------------------------------------------------------

void __fastcall TForm1::N2001Click(TObject *Sender)
{
        N1001->Checked=false;
        N2001->Checked=true;
        N4001->Checked=false;
        UserDefined1->Checked=false;

        ClientWidth=BaseWidth*2;
        ClientHeight=BaseHeight*2;


}
//---------------------------------------------------------------------------

void __fastcall TForm1::N4001Click(TObject *Sender)
{
        N1001->Checked=false;
        N2001->Checked=false;
        N4001->Checked=true;
        UserDefined1->Checked=false;

        ClientWidth=BaseWidth*4;
        ClientHeight=BaseHeight*4;

}
//---------------------------------------------------------------------------

void __fastcall TForm1::UserDefined1Click(TObject *Sender)
{
        N1001->Checked=false;
        N2001->Checked=false;
        N4001->Checked=false;
        UserDefined1->Checked=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Display1Click(TObject *Sender)
{
        PCAllKeysUp();
        Hardware->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Speed1Click(TObject *Sender)
{
        if (Speed1->Checked)
        {
                Speed->Close();
                Speed1->Checked=false;
        }
        else
        {
                PCAllKeysUp();
                Speed->Show();
                Speed1->Checked=true;
        }
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Keyboard1Click(TObject *Sender)
{
        PCAllKeysUp();
        Kb->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::InsertTape1Click(TObject *Sender)
{
        PCAllKeysUp();
        Tape->OpenTape1Click(Sender);
}
//---------------------------------------------------------------------------


void __fastcall TForm1::SaveSnapshot1Click(TObject *Sender)
{
        int stopped;
        stopped=zx81_stop;
        zx81_stop=1;

        if (zx81.machine==MACHINEACE) SaveSnapDialog->Filter = ".ACE Snapshot|*.ace";
        else SaveSnapDialog->Filter = ".Z81 Snapshot|*.z81";
        if (SaveSnapDialog->Execute()) save_snap(SaveSnapDialog->FileName.c_str());
        zx81_stop=stopped;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::LoadSnapshot1Click(TObject *Sender)
{
        int stopped;
        stopped=zx81_stop;

        if (zx81.machine==MACHINEACE) LoadSnapDialog->Filter = ".ACE Snapshot|*.ace";
        else LoadSnapDialog->Filter = ".Z81 Snapshot|*.z81";
        if (!LoadSnapDialog->Execute()) return;

        zx81_stop=1;
        sound_ay_reset();
        load_snap(LoadSnapDialog->FileName.c_str());
        zx81_stop=stopped;
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Tape1Click(TObject *Sender)
{
        if (Tape1->Checked)
        {
                Tape->Close();
                Tape1->Checked=false;
        }
        else
        {
                PCAllKeysUp();
                Tape->Show();
                Tape1->Checked=true;
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CloseTape1Click(TObject *Sender)
{
        PCAllKeysUp();
        Tape->SaveTape1Click(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::NewTape1Click(TObject *Sender)
{
        PCAllKeysUp();
        Tape->NewTape();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Toolbar1Click(TObject *Sender)
{
        Toolbar1->Checked = !Toolbar1->Checked;
        ToolBar->Visible = Toolbar1->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ResetButtonClick(TObject *Sender)
{
        zx81_stop=1;
        zx81_init();
        zx81_stop=0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::InsertTapeButtonClick(TObject *Sender)
{
        PCAllKeysUp();
        Tape->OpenTape1Click(Sender);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CloseTapeButtonClick(TObject *Sender)
{
        PCAllKeysUp();
        Tape->CloseTape();
}
//---------------------------------------------------------------------------


void __fastcall TForm1::ResetZX811Click(TObject *Sender)
{
        //PCAllKeysUp();
        zx81_stop=1;
        z80_reset();
        sound_ay_reset();
        zx81_stop=0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
        TIniFile *ini;

        zx81_stop=true;

        PCAllKeysUp();
        Tape->NewTape();

        sound_end();

        ini = new TIniFile(ChangeFileExt( Application->ExeName, ".ini" ) );
        SaveSettings(ini);
        delete ini;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ViewPrinterClick(TObject *Sender)
{
        ViewPrinter->Checked = !ViewPrinter->Checked;

        if (ViewPrinter->Checked)
        {
                PCAllKeysUp();
                Printer->Show();
        }
        else    Printer->Close();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::InterlacedClick(TObject *Sender)
{
        Interlaced->Checked = !Interlaced->Checked;
        if (Interlaced->Checked && Artifacts)
        {
                Artifacts->ArtEnabled->Checked=false;
                Artifacts->ArtEnabledClick(NULL);
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Timer2Timer(TObject *Sender)
{
        int targetfps;

        if (CommandLine)
        {
                AnsiString Cmd, Ext;

                if (strlen(CommandLine)>3)
                {
                        Cmd = CommandLine;

                        if (Cmd[1]=='\"') Cmd = Cmd.SubString(2,Cmd.Length());
                        if (Cmd[Cmd.Length()]=='\"') Cmd = Cmd.SubString(1,Cmd.Length()-1);

                        Ext = FileNameGetExt(Cmd);

                        if ( (Ext == ".p") || (Ext == ".P") ||(Ext == ".t81") || (Ext==".T81") )
                        {
                                Tape->LoadTapeFile(Cmd);
                                load_tape_buffer(65536);
                        }
                        if ( (Ext == ".z81") || (Ext == ".Z81")
                                || (Ext == ".ace") || (Ext == ".ACE") ) load_snap(Cmd.c_str());
                }
                CommandLine=NULL;
        }

        targetfps = ((zx81.NTSC ? 60:50) * 9) / 10;
        //fps /= 2;

        if (fps > (targetfps+2) && zx81.frameskip>0) zx81.frameskip--;
        if (fps < targetfps && zx81.frameskip<10 && zx81.frameskip>=0) zx81.frameskip++;


        AnsiString text="EightyOne (";

        if (FullScreen) return;

        if (zx81_stop) text +="Paused)";
        else
        {
                if (zx81.single_step) text +="Debug Mode)";
                else
                {
                        text += fps;
                        text += "fps";
                        if (zx81.frameskip>0)
                        {
                                text += ", Frameskip ";
                                text += zx81.frameskip;
                        }
                        text += ")";
                }
        }

        Form1->Caption = text;
        fps=0;

        if (Sound1->Checked && !zx81.vsyncsound) zx81.vsyncsound=1;
        TZX->RecStopCheck();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::WavLoadBtnClick(TObject *Sender)
{
        if (WavLoadBtn->Checked)
        {
                WavLoad->Close();
                WavLoadBtn->Checked=false;
        }
        else
        {
                PCAllKeysUp();
                WavLoad->Show();
                WavLoadBtn->Checked=true;
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormKeyPress(TObject *Sender, char &Key)
{
        extern void RecalcWinSize(void);
        if (Key==27)
        {
                FullScreen = !FullScreen;

                if (FullScreen)
                {
                        SaveX=Left;
                        SaveY=Top;
                        SaveW=ClientWidth;
                        SaveH=ClientHeight;
                        //BorderStyle=bsDialog;

                        DFlipSurface->Width=800;
                        DFlipSurface->Height=600;
                        DFlipSurface->ChangeMode=true;
                        Screen->Cursor = crNone;
                        RecalcWinSize();
                }
                else
                {
                        DFlipSurface->ChangeMode=false;
                        ClientWidth=SaveW;
                        ClientHeight=SaveH;
                        Left=SaveX;
                        Top=SaveY;
                        Screen->Cursor = crDefault;
                        RecalcWinSize();
                        BorderStyle=bsSizeable;
                }

        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::DebugWinClick(TObject *Sender)
{
        DebugWin->Checked = !DebugWin->Checked;
        if (DebugWin->Checked)
                Dbg->Show();
        else
                Dbg->Close();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::AppMessage(TMsg &Msg, bool &Handled)
{
        WORD BufferLength=255;
        AnsiString Filename;
        WORD FileIndex;
        WORD QtyDroppedFiles;
        char pDroppedFilename[255];

        if (Msg.message == WM_DROPFILES)
        {
                QtyDroppedFiles = DragQueryFile((void *)Msg.wParam, -1,
                                                pDroppedFilename, BufferLength);

                for(FileIndex=0; FileIndex<=(QtyDroppedFiles - 1); FileIndex++)
                {
                        DragQueryFile((void *)Msg.wParam, FileIndex, pDroppedFilename, BufferLength);

                        Filename = pDroppedFilename;

                        if (Tape->IsTape(Filename))
                        {
                                Tape->CloseTape();
                                Tape->LoadTapeFile(Filename);
                        }
                        else if (WavLoad->IsWav(Filename))
                        {
                                WavLoad->LoadFile(Filename);
                        }
                        else load_snap(Filename.c_str());
                }

                DragFinish((void *)Msg.wParam);
                Handled = true;
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PauseZX81Click(TObject *Sender)
{
        zx81_stop = !zx81_stop;
        Dbg->UpdateVals();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::AnimTimer1Timer(TObject *Sender)
{
        static int j, borrow;

        if (zx81_stop) return;

        tsmax= frametstates ? frametstates : 65000;
        sound_frame();


        fps++;
        frametstates=0;

        j=zx81.single_step?1:(Speed->AccurateTStates + borrow);

        switch (zx81.machine)
        {
        case MACHINEACE:
                while (j>0)
                {
                        j-= ace_do_accurate();
                        AccurateDraw();
                        if (zx81_stop) return;
                }
                break;

        default:
                while (j>0)
                {
                        j -= zx81_do_accurate();
                        AccurateDraw();
                        if (zx81_stop) return;
                }
                break;
        }
        borrow=j;
        AccurateUpdateDisplay(false);
        DFlipSurface->RepaintTopSurface(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::InverseVideoClick(TObject *Sender)
{
        InverseVideo->Checked = !InverseVideo->Checked;
        zx81.inverse = 1 - zx81.inverse;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormDeactivate(TObject *Sender)
{
        extern void RecalcPalette(void);

        if (FullScreen)
        {
                FullScreen=false;
                ClientWidth=SaveW;
                ClientHeight=SaveH;
                Left=SaveX;
                Top=SaveY;
                Screen->Cursor = crDefault;
                DFlipSurface->ChangeMode=false;
                Form1->WindowState=wsNormal;
                RecalcPalette();
        }
}
//---------------------------------------------------------------------------

void TForm1::LoadSettings(TIniFile *ini)
{
        if (ini->ReadBool("MAIN","N1001",N1001->Checked)) N1001Click(NULL);
        if (ini->ReadBool("MAIN","N2001",N2001->Checked)) N2001Click(NULL);
        if (ini->ReadBool("MAIN","N4001",N4001->Checked)) N4001Click(NULL);
        if (ini->ReadBool("MAIN","UserDefined",UserDefined1->Checked)) UserDefined1Click(NULL);

        //if (ini->ReadBool("MAIN","Fast1",Fast1->Checked)) Fast1Click(NULL);
        //if (ini->ReadBool("MAIN","Accurate1",Accurate1->Checked)) Accurate1Click(NULL);
        if (ini->ReadBool("MAIN","Interlaced",Interlaced->Checked)) InterlacedClick(NULL);

        //if (ini->ReadBool("MAIN","DisplayArtifacts",DisplayArtifacts1->Checked)) DisplayArtifacts1Click(NULL);
        if (ini->ReadBool("MAIN","InverseVideo",InverseVideo->Checked)) InverseVideoClick(NULL);
        if (ini->ReadBool("MAIN","Toolbar1",Toolbar1->Checked)) Toolbar1Click(NULL);

        Keyboard1->Checked = ini->ReadBool("MAIN", "Keyboard1", Keyboard1->Checked);
        Display1->Checked = ini->ReadBool("MAIN", "Display1", Display1->Checked);
        Speed1->Checked = ini->ReadBool("MAIN", "Speed1", Speed1->Checked);

        DisplayArt->Checked=ini->ReadBool("MAIN","DisplayArt",DisplayArt->Checked);

        DebugWin->Checked = ini->ReadBool("MAIN", "DebugWin", DebugWin->Checked);
        ViewPrinter->Checked = ini->ReadBool("MAIN", "ViewPrinter", ViewPrinter->Checked);
        Tape1->Checked = ini->ReadBool("MAIN", "Tape1", Tape1->Checked);
        WavLoadBtn->Checked = ini->ReadBool("MAIN", "WavLoadBtn", WavLoadBtn->Checked);
        KeyboardMap1->Checked = ini->ReadBool("MAIN", "KeyMap", KeyboardMap1->Checked);
        TZXMan->Checked = ini->ReadBool("MAIN", "TZXManager", TZXMan->Checked);

        InWaveLoader->Checked = ini->ReadBool("MAIN", "InWave", InWaveLoader->Checked);
        InTapeManager->Checked = ini->ReadBool("MAIN", "InTape", InTapeManager->Checked);
        InTZXManager->Checked = ini->ReadBool("MAIN", "InTZX", InTZXManager->Checked);
        OutWaveLoader->Checked = ini->ReadBool("MAIN", "OutWave", OutWaveLoader->Checked);
        OutTapeManager->Checked = ini->ReadBool("MAIN", "OutTape", OutTapeManager->Checked);
        OutTZXManager->Checked = ini->ReadBool("MAIN", "OutTZX", OutTZXManager->Checked);
        OutAudioOut->Checked = ini->ReadBool("MAIN", "OutAudio", OutAudioOut->Checked);

        zx81.interceptload = InTapeManager->Checked ? 1:0;
        zx81.interceptsave = OutTapeManager->Checked ? 1:0;
        zx81.audioout = OutAudioOut->Checked ? 1:0;
        zx81.TZXin = InTZXManager->Checked ? 1:0;
        zx81.TZXout = OutTZXManager->Checked ? 1:0;

        Top = ini->ReadInteger("MAIN","Top",Top);
        Left = ini->ReadInteger("MAIN","Left",Left);
        Height = ini->ReadInteger("MAIN","Height",Height);
        Width = ini->ReadInteger("MAIN","Width",Width);
}

void TForm1::SaveSettings(TIniFile *ini)
{
        ini->WriteInteger("MAIN","Top",Top);
        ini->WriteInteger("MAIN","Left",Left);
        ini->WriteInteger("MAIN","Height",Height);
        ini->WriteInteger("MAIN","Width",Width);

        ini->WriteBool("MAIN","N1001",N1001->Checked);
        ini->WriteBool("MAIN","N2001",N2001->Checked);
        ini->WriteBool("MAIN","N4001",N4001->Checked);
        ini->WriteBool("MAIN","UserDefined",UserDefined1->Checked);

        //ini->WriteBool("MAIN","Fast1",Fast1->Checked);
        //ini->WriteBool("MAIN","Accurate1",Accurate1->Checked);
        ini->WriteBool("MAIN","Interlaced",Interlaced->Checked);
        ini->WriteBool("MAIN","InverseVideo",InverseVideo->Checked);
        ini->WriteBool("MAIN","DisplayArt",DisplayArt->Checked);
        ini->WriteBool("MAIN","Toolbar1",Toolbar1->Checked);

        ini->WriteBool("MAIN", "Keyboard1", Keyboard1->Checked);
        ini->WriteBool("MAIN", "Display1", Display1->Checked);
        ini->WriteBool("MAIN", "Speed1", Speed1->Checked);
        ini->WriteBool("MAIN", "InWave", InWaveLoader->Checked);
        ini->WriteBool("MAIN", "InTape", InTapeManager->Checked);
        ini->WriteBool("MAIN", "InTZX", InTZXManager->Checked);
        ini->WriteBool("MAIN", "OutWave", OutWaveLoader->Checked);
        ini->WriteBool("MAIN", "OutTape", OutTapeManager->Checked);
        ini->WriteBool("MAIN", "OutTZX", OutTZXManager->Checked);
        ini->WriteBool("MAIN", "OutAudio", OutAudioOut->Checked);


        ini->WriteBool("MAIN", "DebugWin", DebugWin->Checked);
        ini->WriteBool("MAIN", "ViewPrinter", ViewPrinter->Checked);
        ini->WriteBool("MAIN", "Tape1", Tape1->Checked);
        ini->WriteBool("MAIN", "WavLoadBtn", WavLoadBtn->Checked);
        ini->WriteBool("MAIN", "TZXManager", TZXMan->Checked);

        ini->WriteBool("MAIN", "KeyMap", KeyboardMap1->Checked);

        Keyboard->SaveSettings(ini);
        Speed->SaveSettings(ini);
        Kb->SaveSettings(ini);
        Printer->SaveSettings(ini);
        WavLoad->SaveSettings(ini);
        Dbg->SaveSettings(ini);
        Hardware->SaveSettings(ini);
        Tape->SaveSettings(ini);
        Artifacts->SaveSettings(ini);
        SerialConfig->SaveSettings(ini);
        TZX->SaveSettings(ini);

}

void __fastcall TForm1::DisplayArtClick(TObject *Sender)
{
        DisplayArt->Checked = !DisplayArt->Checked;
        if (DisplayArt->Checked) Artifacts->Show();
        else Artifacts->Close();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Sound1Click(TObject *Sender)
{
        Sound1->Checked = !Sound1->Checked;
        zx81.vsyncsound = Sound1->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::DBG1Click(TObject *Sender)
{
        DBG1->Checked=!DBG1->Checked;
        zx81.debug1=DBG1->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::HelpTopics2Click(TObject *Sender)
{
        ShellExecute(0,NULL, "eightyone.chm", NULL, NULL, SW_SHOW);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::InTapeManagerClick(TObject *Sender)
{
        InWaveLoader->Checked=false;
        InTZXManager->Checked=false;
        InTapeManager->Checked=true;
        zx81.interceptload=1;
        zx81.TZXin=0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::InWaveLoaderClick(TObject *Sender)
{
        InWaveLoader->Checked=true;
        InTapeManager->Checked=false;
        InTZXManager->Checked=false;
        zx81.interceptload=0;
        zx81.TZXin=0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OutTapeManagerClick(TObject *Sender)
{
        OutWaveLoader->Checked=false;
        OutTapeManager->Checked=true;
        OutAudioOut->Checked=false;
        zx81.interceptsave=1;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OutWaveLoaderClick(TObject *Sender)
{
        OutWaveLoader->Checked=true;
        OutTapeManager->Checked=false;
        OutAudioOut->Checked=false;
        zx81.interceptsave=0;
        zx81.audioout=0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OutAudioOutClick(TObject *Sender)
{
        OutWaveLoader->Checked=false;
        OutTapeManager->Checked=false;
        OutAudioOut->Checked=true;
        zx81.interceptsave=0;
        zx81.audioout=1;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SoundOutput1Click(TObject *Sender)
{
        SoundOutput1->Checked = !SoundOutput1->Checked;
        if (SoundOutput1->Checked) SoundOutput->Show();
        else SoundOutput->Close();

}
//---------------------------------------------------------------------------

void __fastcall TForm1::ROMClick(TObject *Sender)
{
        PCAllKeysUp();
        ROMBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::LoadMemoryBlock1Click(TObject *Sender)
{
        MemSave->LoadMem();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SaveMemoryBlock1Click(TObject *Sender)
{
        MemSave->SaveMem();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::HardReset1Click(TObject *Sender)
{
        //PCAllKeysUp();
        zx81_stop=1;
        z80_reset();
        zx81_init();
        sound_ay_reset();
        zx81_stop=0;

}
//---------------------------------------------------------------------------

void __fastcall TForm1::Zx97ConfigClick(TObject *Sender)
{
        Form1->Zx97Config->Checked = !Form1->Zx97Config->Checked;

        if (Form1->Zx97Config->Checked)
                ZX97Dialog->Show();
        else
                ZX97Dialog->Close();

}
//---------------------------------------------------------------------------

void __fastcall TForm1::TS20501Click(TObject *Sender)
{
        SerialConfig->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::TZXManClick(TObject *Sender)
{
        Form1->TZXMan->Checked = !Form1->TZXMan->Checked;

        if (Form1->TZXMan->Checked)
                TZX->Show();
        else
                TZX->Close();

}
//---------------------------------------------------------------------------

void __fastcall TForm1::InTZXManagerClick(TObject *Sender)
{
        InWaveLoader->Checked=false;
        InTZXManager->Checked=true;
        InTapeManager->Checked=false;
        zx81.TZXin=1;
        zx81.interceptload=0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OutTZXManagerClick(TObject *Sender)
{
        OutWaveLoader->Checked=false;
        OutTZXManager->Checked=true;
        OutTapeManager->Checked=false;
        OutAudioOut->Checked=false;
        zx81.TZXout=1;
        zx81.interceptsave=0;
        zx81.audioout=0;
}
//---------------------------------------------------------------------------


void __fastcall TForm1::RPWobble1Click(TObject *Sender)
{
        RPWobble1->Checked = !RPWobble1->Checked;
        zx81.wobble = RPWobble1->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::WobbleNow1Click(TObject *Sender)
{
        ramwobble(true);        
}
//---------------------------------------------------------------------------

