//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "TZXMan.h"
#include "tzxfile.h"
#include "main_.h"
#include "utils.h"

TTZXFile TZXFile;

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTZX *TZX;
//---------------------------------------------------------------------------
void TZXClockTick(int TStates)
{
        static bool Previous=true;

        if (TZXFile.ClockTick(TStates))
        {
                int a;
                a=TZXFile.CurBlock+1;
                TZX->Table->Row=a;
                TZXFile.CurBlock=a-1;
        }

        if (TZXFile.Playing!=Previous)
        {
                Previous=TZXFile.Playing;
                if (Previous) TZX->Caption = "TZX (Playing)";
                else TZX->Caption = "TZX (Stopped)";
        }
}

int TZXGetEarState(void)
{
        return(TZXFile.GetEarState());
}

void TZXStartPlaying(void)
{
        if (TZX->AutoStartStop->Checked)
                TZX->PlayBtnClick(NULL);
}

void TZXWriteByte(unsigned char Byte)
{
        TZX->RecordByte(Byte);
}
//---------------------------------------------------------------------------

void TTZX::AddBlock(char *buffer, int len)
{
        if (!buffer)    TZXFile.AddPauseBlock(len/3250);
        else    TZXFile.AddGeneralBlock(buffer,len);
        UpdateTable();
}

void TTZX::UpdateTable(void)
{
        int i;

        Table->RowCount=TZXFile.Blocks+2;

        for(i=0;i<TZXFile.Blocks;i++)
        {
                Table->Cells[0][i+1]=TZXFile.GetBlockType(i);
                Table->Cells[1][i+1]=TZXFile.GetBlockName(i);
                Table->Cells[2][i+1]=TZXFile.GetBlockLength(i);
        }

        Table->Rows[TZXFile.Blocks+1]->CommaText=",,";
        TZXFile.CurBlock=Table->Row-1;
}
//---------------------------------------------------------------------------
__fastcall TTZX::TTZX(TComponent* Owner)
        : TForm(Owner)
{
        TIniFile *ini;

        ini = new TIniFile(ChangeFileExt(Application->ExeName, ".INI" ));
        LoadSettings(ini);
        delete ini;


        Table->Rows[0]->CommaText="Type,Description,Size";
}
//---------------------------------------------------------------------------

void __fastcall TTZX::Open1Click(TObject *Sender)
{
        int i;

        if (!OpenDialog->Execute()) return;
        if (!TZXFile.LoadFile(OpenDialog->FileName)) return;

        UpdateTable();
        Table->Row=1;
        TZXFile.CurBlock=Table->Row-1;

}
//---------------------------------------------------------------------------
void __fastcall TTZX::PlayBtnClick(TObject *Sender)
{
        TZXFile.Start();
}
//---------------------------------------------------------------------------

void __fastcall TTZX::StopBtnClick(TObject *Sender)
{
        TZXFile.Stop(true);
}
//---------------------------------------------------------------------------

void __fastcall TTZX::FormResize(TObject *Sender)
{
        Table->ColWidths[1]=TZX->ClientWidth- (Table->ColWidths[0]+Table->ColWidths[2]);
}
//---------------------------------------------------------------------------

void __fastcall TTZX::TableSelectCell(TObject *Sender, int ACol, int ARow,
      bool &CanSelect)
{
        int line;
        //TZXFile.Stop();
        TZXFile.CurBlock=Table->Row-1;

        line=Table->Row;
        if (line<1 || line>TZXFile.Blocks)
        {
                EditBlock2->Enabled=false;
                InsertBlock2->Enabled=false;
                ExtractBlock2->Enabled=false;
                MoveBlockUp2->Enabled=false;
                MoveBlockDown2->Enabled=false;
                DeleteBlock2->Enabled=false;
        }
        else
        {
                if (TZXFile.IsEditable(line-1)) EditBlock2->Enabled=true;
                else EditBlock2->Enabled=false;

                //EditBlock1->Enabled=true;
                //InsertBlock1->Enabled=true;
                //ExtractBlock1->Enabled=true;
                MoveBlockUp2->Enabled=true;
                MoveBlockDown2->Enabled=true;
                DeleteBlock2->Enabled=true;
        }

        if (line==TZXFile.Blocks+1)
        {
                Table->Row=line;
                InsertBlock2->Enabled=true;
        }

        if (line==TZXFile.Blocks) MoveBlockDown2->Enabled=false;
        if (line==1) MoveBlockUp2->Enabled=false;
        if (Table->Cells[0][line] == "25") ExtractBlock2->Enabled=true;
        else ExtractBlock2->Enabled=false;
}
//---------------------------------------------------------------------------
void TTZX::MergeBlocks(void)
{
        TZXFile.MergeBlocks();
}

//---------------------------------------------------------------------------
void TTZX::LoadSettings(TIniFile *ini)
{
        Top = ini->ReadInteger("TZXMAN","Top",Top);
        Left = ini->ReadInteger("TZXMAN","Left",Left);
        Height = ini->ReadInteger("TZXMAN","Height",Height);
        Width = ini->ReadInteger("TZXMAN","Width",Width);

        OpenDialog->FileName = ini->ReadString("TZXMAN","Filename",OpenDialog->FileName);
        OpenDialog->InitialDir = ini->ReadString("TZXMAN","Dir",OpenDialog->InitialDir);

        FlashLoad1->Checked = ini->ReadBool("TZXMAN","FlashLoad",FlashLoad1->Checked);
        AutoStartStop->Checked = ini->ReadBool("TZXMAN","AutoStart",AutoStartStop->Checked);

        TZXFile.AutoStart=AutoStartStop->Checked;
        TZXFile.FlashLoad=FlashLoad1->Checked;
        if (Form1->TZXMan->Checked) Show();

        //TZXFile.NewTZX();
        NewTZXClick(NULL);
        UpdateTable();
}

void TTZX::SaveSettings(TIniFile *ini)
{
        ini->WriteInteger("TZXMAN","Top",Top);
        ini->WriteInteger("TZXMAN","Left",Left);
        ini->WriteInteger("TZXMAN","Height",Height);
        ini->WriteInteger("TZXMAN","Width",Width);

        ini->WriteString("TZXMAN","Filename",OpenDialog->FileName);
        ini->WriteString("TZXMAN","Dir",OpenDialog->InitialDir);
        ini->WriteBool("TZXMAN","FlashLoad",FlashLoad1->Checked);
        ini->WriteBool("TZXMAN","AutoStart",AutoStartStop->Checked);
}
//---------------------------------------------------------------------------


void __fastcall TTZX::NewTZXClick(TObject *Sender)
{
        TZXFile.NewTZX();
        UpdateTable();
        Table->Row=3;
}
//---------------------------------------------------------------------------

void __fastcall TTZX::SaveAs1Click(TObject *Sender)
{
        if (TZXFile.FileName != "") SaveDialog->FileName=TZXFile.FileName;
        if (!SaveDialog->Execute()) return;
        TZXFile.SaveFile(SaveDialog->FileName);
}
//---------------------------------------------------------------------------

void __fastcall TTZX::FlashLoad1Click(TObject *Sender)
{
        FlashLoad1->Checked = !FlashLoad1->Checked;
        TZXFile.FlashLoad=FlashLoad1->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TTZX::FormClose(TObject *Sender, TCloseAction &Action)
{
        Form1->TZXMan->Checked=false;        
}
//---------------------------------------------------------------------------

void __fastcall TTZX::TableContextPopup(TObject *Sender, TPoint &MousePos,
      bool &Handled)
{
        int line;
        Mx=Left+MousePos.x;
        My=32+Top+MousePos.y;

        line=Table->TopRow + MousePos.y / (Table->DefaultRowHeight+Table->GridLineWidth) - 1;
        if (line<1 || line>TZXFile.Blocks)
        {
                EditBlock1->Enabled=false;
                InsertBlock1->Enabled=false;
                ExtractBlock1->Enabled=false;
                MoveBlockUp1->Enabled=false;
                MoveBlockDown1->Enabled=false;
                DeleteBlock1->Enabled=false;
        }
        else
        {
                Table->Row=line;

                if (TZXFile.IsEditable(line-1)) EditBlock1->Enabled=true;
                else EditBlock1->Enabled=false;

                //EditBlock1->Enabled=true;
                //InsertBlock1->Enabled=true;
                //ExtractBlock1->Enabled=true;
                MoveBlockUp1->Enabled=true;
                MoveBlockDown1->Enabled=true;
                DeleteBlock1->Enabled=true;
        }

        if (line==TZXFile.Blocks+1)
        {
                Table->Row=line;
                InsertBlock1->Enabled=true;
        }

        if (line==TZXFile.Blocks) MoveBlockDown1->Enabled=false;
        if (line==1) MoveBlockUp1->Enabled=false;
        if (Table->Cells[0][line] == "25") ExtractBlock1->Enabled=true;
        else ExtractBlock1->Enabled=false;

}
//---------------------------------------------------------------------------

void __fastcall TTZX::MoveBlockUp1Click(TObject *Sender)
{
        TZXFile.SwapBlocks(Table->Row-1, Table->Row-2);
        UpdateTable();
}
//---------------------------------------------------------------------------

void __fastcall TTZX::MoveBlockDown1Click(TObject *Sender)
{
        TZXFile.SwapBlocks(Table->Row, Table->Row-1);
        UpdateTable();
}
//---------------------------------------------------------------------------

void __fastcall TTZX::DeleteBlock1Click(TObject *Sender)
{
        TZXFile.DeleteBlock(Table->Row-1);
        UpdateTable();
}
//---------------------------------------------------------------------------

void __fastcall TTZX::EditBlock1Click(TObject *Sender)
{
        Mx = TZX->Left + Table->Left + (Table->Width/2);
        My = TZX->Top + Table->Top + (
                Table->Row - Table->TopRow)*(Table->DefaultRowHeight*Table->GridLineWidth);

        TZXFile.EditBlock(Table->Row-1, Mx, My);
        UpdateTable();
}
//---------------------------------------------------------------------------

void __fastcall TTZX::AutoStartStopClick(TObject *Sender)
{
        AutoStartStop->Checked = !AutoStartStop->Checked;
        TZXFile.AutoStart = AutoStartStop->Checked;
}
//---------------------------------------------------------------------------


void __fastcall TTZX::HardwareInformation1Click(TObject *Sender)
{
        TZXFile.MoveBlock(TZXFile.AddHWTypeBlock(0,0x0c),Table->Row-1);
        TZXFile.EditBlock(Table->Row-1, Mx, My);
        UpdateTable();

}
//---------------------------------------------------------------------------

void __fastcall TTZX::ArchiveInfo1Click(TObject *Sender)
{
        TZXFile.MoveBlock(TZXFile.AddArchiveBlock("Title"),Table->Row-1);
        TZXFile.EditBlock(Table->Row-1, Mx, My);
        UpdateTable();

}
//---------------------------------------------------------------------------

void __fastcall TTZX::TextDescription1Click(TObject *Sender)
{
        TZXFile.MoveBlock(TZXFile.AddTextBlock("Message"),Table->Row-1);
        TZXFile.EditBlock(Table->Row-1, Mx, My);
        UpdateTable();

}
//---------------------------------------------------------------------------

void __fastcall TTZX::Pause1Click(TObject *Sender)
{
        TZXFile.MoveBlock(TZXFile.AddPauseBlock(0),Table->Row-1);
        TZXFile.EditBlock(Table->Row-1, Mx, My);
        UpdateTable();

}
//---------------------------------------------------------------------------
#define REC_STOP 0
#define REC_RECORDING 1
#define REC_ENDING 2

void TTZX::RecordByte(unsigned char c)
{
        if (RecState==REC_STOP)
        {
                RecPtr=RecBuf;
                RecState=REC_RECORDING;
        }

        if (RecState==REC_RECORDING || RecState==REC_ENDING)
        {
                *RecPtr=c;
                RecPtr++;
                RecState=REC_RECORDING;
        }
}

void TTZX::RecStopCheck(void)
{
        if (RecState==REC_STOP) return;

        if (RecState==REC_RECORDING)
        {
                RecState=REC_ENDING;
                return;
        }

        RecState=REC_STOP;
        TZXFile.MoveBlock(TZXFile.AddGeneralBlock(RecBuf, RecPtr - RecBuf),Table->Row-1);
        UpdateTable();
        Table->Row ++;
}


void __fastcall TTZX::Data1Click(TObject *Sender)
{
        AnsiString FName, Ext;
        unsigned char buf[65536], *p;
        FILE *f;

        if (!InsertBlockDialog1->Execute()) return;

        FName=InsertBlockDialog1->FileName;

        p=buf;
        if (FileNameGetExt(FName)==".P")
        {
                ConvertASCIIZX81(FileNameGetFname(FName), buf);
                while(!((*p)&128)) p++; p++;
        }

        f=fopen(FName.c_str(),"rb");
        if (!f) return;
        p += fread(p, 1, 65535, f);
        fclose(f);

        TZXFile.MoveBlock(TZXFile.AddGeneralBlock(buf, p-buf),Table->Row-1);
        UpdateTable();
        Table->Row ++;
}
//---------------------------------------------------------------------------


void __fastcall TTZX::ExtractBlock1Click(TObject *Sender)
{
        unsigned char *Data, *p;
        int DataLen;
        int FNameLen;
        FILE *f;

        Data=TZXFile.Tape[Table->Row-1].Data.Data;
        DataLen=TZXFile.Tape[Table->Row-1].Head.General.DataLen;

        FNameLen=0;
        p=Data;

        while( !((*p)&128) && FNameLen<64){ p++; FNameLen++; } p++;

        if (FNameLen>0 && FNameLen<=32)
        {
                ExtractDialog1->FilterIndex=1;
                ExtractDialog1->FileName=ConvertZX81ASCII(Data);
        }
        else
        {
                ExtractDialog1->FilterIndex=2;
                ExtractDialog1->FileName="code";
                p=Data;
        }

        if (!ExtractDialog1->Execute()) return;

        f=fopen(ExtractDialog1->FileName.c_str(), "wb");
        if (!f) return;

        fwrite(p, 1, DataLen - (p-Data), f);
        fclose(f);
}
//---------------------------------------------------------------------------

void __fastcall TTZX::Save1Click(TObject *Sender)
{
        if (TZXFile.FileName=="") SaveAs1Click(NULL);
        else TZXFile.SaveFile(SaveDialog->FileName);
}
//---------------------------------------------------------------------------

