//---------------------------------------------------------------------------

#include <vcl.h>
#include <stdio.h>
#pragma hdrstop

#include "tzxfile.h"
#include "tzxEditPause.h"
#include "tzxEditArchive.h"
#include "tzxEditText.h"
#include "tzxEditHWInfo.h"
#include "tzxEditGeneral.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

char *HWName[]=
{
        "ZX Spectrum 16k",
        "ZX Spectrum 48k, Plus",
        "ZX Spectrum 48k ISSUE 1",
        "ZX Spectrum 128k +(Sinclair)",
        "ZX Spectrum 128k +2 (grey case)",
        "ZX Spectrum 128k +2A, +3",
        "Timex Sinclair TC-2048",
        "Timex Sinclair TS-2068",
        "Pentagon 128",
        "Sam Coupe",
        "Didaktik M",
        "Didaktik Gama",
        "ZX-81",
        "ZX-80",
        "ZX Spectrum 128k, Spanish version",
        "ZX Spectrum, Arabic version",
        "TK 90-X",
        "TK 95",
        "Byte",
        "Elwro",
        "ZS Scorpion",
        "Amstrad CPC 464",
        "Amstrad CPC 664",
        "Amstrad CPC 6128",
        "Amstrad CPC 464+",
        "Amstrad CPC 6128+",
        "Jupiter ACE",
        "Enterprise",
        "Commodore 64",
        "Commodore 128",
        "TS1500",
        "Lambda",
        "TK85",
        "ZX97"
};

struct BLKNAMES
{
        int id;
        char *name;
} BlockNames[] =
{       { 0,                      "Unknown" },
        { TZX_BLOCK_ROM,          "Spectrum ROM" },
        { TZX_BLOCK_TURBO,        "Turbo Loader" },
        { TZX_BLOCK_TONE,         "Pure Tone" },
        { TZX_BLOCK_PULSE,        "Pulse Sequence" },
        { TZX_BLOCK_DATA,         "Pure Data" },
        { TZX_BLOCK_DREC,         "Direct Recording" },
        { TZX_BLOCK_CSW,          "CSW Recording" },
        { TZX_BLOCK_GENERAL,      "General Data" },
        { TZX_BLOCK_PAUSE,        "Pause" },
        { TZX_BLOCK_GSTART,       "Group Start" },
        { TZX_BLOCK_GEND,         "Group End" },
        { TZX_BLOCK_JUMP,         "Jump to Block" },
        { TZX_BLOCK_LSTART,       "Loop Start" },
        { TZX_BLOCK_LEND,         "Loop End" },
        { TZX_BLOCK_SBLOCK,       "Select Block" },
        { TZX_BLOCK_STOP48K,      "Stop Tape" },
        { TZX_BLOCK_SETLEVEL,     "Set Level" },
        { TZX_BLOCK_TEXT,         "Text Description" },
        { TZX_BLOCK_MESSAGE,      "Message" },
        { TZX_BLOCK_ARCHIVE,      "Archive info" },
        { TZX_BLOCK_HWTYPE,       "Hardware Type" },
        { TZX_BLOCK_CUSTOM,       "Custom Info" },
        { TZX_BLOCK_GLUE,         "Glue" },
        { -1,                     "" }
} ;

void TTZXFile::EraseAll(void)
{
        int i;
        for(i=0;i<Blocks;i++) EraseBlock(i);
        Blocks=CurBlock=0;
}

void TTZXFile::NewTZX(void)
{
        EraseAll();
        AddTextBlock("Created with EightyOneTZX");
        AddHWTypeBlock(0x00, 0x0c);
        CurBlock=2;
}

void TTZXFile::EraseBlock(int BlockNo)
{
        Tape[BlockNo].BlockID=0;

        if (Tape[BlockNo].Data.Pulses)
        {
                free(Tape[BlockNo].Data.Pulses);
                Tape[BlockNo].Data.Pulses=NULL;
        }

        if (Tape[BlockNo].SyncPulses)
        {
                free(Tape[BlockNo].SyncPulses);
                Tape[BlockNo].SyncPulses=NULL;
        }

        if (Tape[BlockNo].Alphabet)
        {
                free(Tape[BlockNo].Alphabet);
                Tape[BlockNo].Alphabet=NULL;
        }
}

AnsiString TTZXFile::GetBlockName(int BlockNo)
{
        int i=0, len, BlockID;
        AnsiString text;
        char c;

        BlockID=Tape[BlockNo].BlockID;

        switch(BlockID)
        {
        case 0x30:
        case 0x21:
                len=Tape[BlockNo].Head.Text.TextLen;

                text="";
                for(i=0;i<len;i++)
                {
                        c=Tape[BlockNo].Data.Data[i];
                        text += c;
                }

                return(text);

        case 0x33:
                text="Hardware - ";
                text += HWName[Tape[BlockNo].Data.HWTypes[0].ID];
                return(text);

        case 0x19:
                text=GetFName(BlockNo);
                if (text!="") text = "Program: "+text;
                else text="Code";
                return(text);

        default:
                do
                {
                        if (BlockNames[i].id==Tape[BlockNo].BlockID)
                                return(BlockNames[i].name);

                                i++;
                } while (BlockNames[i].id != -1);
        }

        return("");
}

unsigned char *TTZXFile::GetBlockData(int Block)
{
        return(Tape[Block].Data.Data);
}

void TTZXFile::DeleteBlock(int Block)
{
        int i;

        if (Block>=Blocks) return;
        EraseBlock(Block);

        for(i=Block; i<=Blocks; i++)
                Tape[i]=Tape[i+1];

        Blocks--;
}

void TTZXFile::MoveBlock(int from, int to)
{
        while(from!=to)
        {
                if (from==to) return;
                if (from>to)
                {
                        SwapBlocks(from, from-1);
                        from--;
                }
                else
                {
                        SwapBlocks(from, from+1);
                        from++;
                }
        }
}

void TTZXFile::SwapBlocks(int b1, int b2)
{
        struct TZXBlock b;

        b=Tape[b1];
        Tape[b1]=Tape[b2];
        Tape[b2]=b;
}

void TTZXFile::MergeBlocks(void)
{
        int i;
        if (!Blocks) return;

        for(i=0;i<Blocks;i++)
        {
                while((Tape[i].BlockID==TZX_BLOCK_GENERAL
                        || Tape[i].BlockID==TZX_BLOCK_PAUSE)
                        && Tape[i+1].BlockID==TZX_BLOCK_PAUSE)
                {
                        Tape[i].Pause += Tape[i+1].Pause;
                        DeleteBlock(i+1);
                }
        }

        i=0;
        while(Tape[i].BlockID==TZX_BLOCK_TEXT
                || Tape[i].BlockID==TZX_BLOCK_MESSAGE
                || Tape[i].BlockID==TZX_BLOCK_ARCHIVE
                || Tape[i].BlockID==TZX_BLOCK_HWTYPE) i++;

        if (Tape[i].BlockID==TZX_BLOCK_PAUSE) DeleteBlock(i);
}

bool TTZXFile::IsEditable(int BlockNo)
{
        switch(Tape[BlockNo].BlockID)
        {
        case 0x30:
        case 0x33:
        case 0x19:
        case 0x20:
        case 0x32:
                return(true);

        default:
                return(false);
        }
}

AnsiString TTZXFile::GetBlockType(int BlockNo)
{
        switch(Tape[BlockNo].BlockID)
        {
        case 0x30:
        case 0x33:
                return("Info");

        default:
                return(Tape[BlockNo].BlockID);
        }
        //return(Tape[BlockNo].BlockID);
}

AnsiString TTZXFile::GetBlockLength(int BlockNo)
{
        AnsiString value;
        int len=-1;

        switch (Tape[BlockNo].BlockID)
        {
        case TZX_BLOCK_PAUSE:
                        len=-Tape[BlockNo].Pause;
                        break;
        case TZX_BLOCK_ROM:
                        len=Tape[BlockNo].Head.ROM.DataLen;
                        break;
        case TZX_BLOCK_TURBO:
                        len=Tape[BlockNo].Head.Turbo.DataLen;
                        break;
        case TZX_BLOCK_DATA:
                        len=Tape[BlockNo].Head.Data.DataLen;
                        break;
        case TZX_BLOCK_GENERAL:
                        len=Tape[BlockNo].Head.General.DataLen;
                        break;
        case TZX_BLOCK_TONE:
        case TZX_BLOCK_PULSE:
        case TZX_BLOCK_DREC:
        case TZX_BLOCK_CSW:
        default:
                break;
        }

        if (len==-1) return("");

        if (len>=0) return(len);

        len=-len;
        value=len;

        return(value + "ms");
}

bool TTZXFile::GetEarState(void)
{
        return(EarState!=0);
}

AnsiString TTZXFile::GetFName(int BlockNo)
{
        AnsiString Name="";
        char *p,c;
        int i=32;
        bool end=false;


        if (Tape[BlockNo].BlockID != TZX_BLOCK_GENERAL) return("");

        p=Tape[BlockNo].Data.Data;

        do
        {
                c=*(p++);

                if (c&128)
                {
                        end=true;
                        c=c&127;
                }

                if (c==0) Name += " ";
                if (c>=28 && c<=37) Name += (char)((c-28)+('0'));
                if (c>=38 && c<=63) Name += (char)((c-38)+('A'));
                i--;
        } while(i && !end);

        return(Name);
}

void TTZXFile::EditBlock(int Block, int Mx, int My)
{
        switch(Tape[Block].BlockID)
        {
        case 0x20:
                EditPauseForm->Go(Block, Mx, My);
                break;

        case 0x32:
                EditArchiveInfo->Go(Block, Mx, My);
                break;

        case 0x30:
                EditTextForm->Go(Block, Mx, My);
                break;

        case 0x33:
                EditHWInfoForm->Go(Block, Mx, My);
                break;
        case 0x19:
                EditGeneralForm->Go(Block, Mx, My);
                break;
        default:
                break;
        }
        return;
}

