//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "wavload_.h"
#include "main_.h"
#include "utils.h"
#include "tapeman_.h"
#include "zx81config.h"
#include "TZXMan.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TWavLoad *WavLoad;
TWavFile Wav;

int GLOBALBYTE;

#define  TSClick 4


bool TWavLoad::IsWav(AnsiString FileName)
{
        if (FileNameGetExt(FileName)==".WAV") return(true);
        return(false);
}

void TWavLoad::ClearImage()
{
        TCanvas *Img;
        TRect rect;
        int w,h,x,y;


        Img=Image1->Canvas;
        w= Image1->Width;
        h=Image1->Height;

        rect.Top=0; rect.Left=0;
        rect.Right=w; rect.Bottom=h;

        Img->Pen->Color = clWhite;

        Img->FillRect(rect);
        Img->Pen->Color = clRed;
        Img->MoveTo(0,h/2);
        Img->LineTo(w,h/2);
        Img->Pen->Color = clGreen;
        Img->MoveTo(0,64-(Threshold/2));
        Img->LineTo(w,64-(Threshold/2));
        DoCaption("");
}

void TWavLoad::UpdateImage()
{
        TCanvas *Img;
        TRect rect;
        long w,h,x,y;
        int Pos,Sample;

        Img=Image1->Canvas;
        Pos=ScrollBar->Position;

        w= Image1->Width;
        h=Image1->Height;

        rect.Top=0; rect.Left=0;
        rect.Right=w; rect.Bottom=h;

        Img->Pen->Color = clWhite;
        Img->FillRect(rect);

        Img->Pen->Color = clRed;
        Img->MoveTo(0,h/2);
        Img->LineTo(w,h/2);
        Img->Pen->Color = clGreen;
        Img->MoveTo(0,64-(Threshold/2));
        Img->LineTo(w,64-(Threshold/2));

        Img->Pen->Color = clBlack;
        Img->MoveTo(0, Wav.Sample(Pos,Channel)/2);
        for (x=0; x<w; x++)
        {
                //Img->LineTo(x,255-Wav.Sample(Pos+x));
                //Img->MoveTo(x,64);
                Sample=Wav.Sample(Pos+x,Channel);


                Img->LineTo(x, Sample/2);

                if  (    (((128-Sample) > Threshold) && Threshold>=0)
                      || (((128-Sample) < Threshold) && Threshold<0)  )
                {
                        Img->MoveTo(x,64-(Threshold/2));
                        Img->LineTo(x, Wav.Sample(Pos+x,Channel)/2);
                }
        }
        DoCaption("");
}
//---------------------------------------------------------------------------
__fastcall TWavLoad::TWavLoad(TComponent* Owner)
        : TForm(Owner)
{
        TIniFile *ini;

        DragAcceptFiles(Handle, true);
        TStatesSample=0;
        TapePos=0;
        CurrentTStates=0;
        ScreenCounter=0;
        Playing=0;
        Threshold=12;
        Channel=0;
        StatusText="Stopped";
        UseLeftChannel1->Enabled=false;
        UseRightChannel1->Enabled=false;
        UseLeftChannel1->Checked=false;
        UseRightChannel1->Checked=false;

        IgnoreZX81=false;

        ini = new TIniFile(ChangeFileExt(Application->ExeName, ".INI" ));
        LoadSettings(ini);
        delete ini;

        NewClick(this);
        ClearImage();
}
//---------------------------------------------------------------------------

void TWavLoad::LoadFile(AnsiString FName)
{
        DoCaption("Loading...");
        Wav.LoadFile(FName);

        FileName=FName;

        TStatesSample=((3250000 / Wav.SampleRate));//*7)/8;
        TapePos=0;
        CurrentTStates=0;
        Playing=0;
        Channel=0;
        ScrollBar->Max = Wav.NoSamples;
        ScrollBar->Position = 0;

        UseLeftChannel1->Enabled=false;
        UseRightChannel1->Enabled=false;
        UseLeftChannel1->Checked=false;
        UseRightChannel1->Checked=false;
        if (Wav.Stereo)
        {
                UseLeftChannel1->Enabled=true;
                UseRightChannel1->Enabled=true;
                UseLeftChannel1->Checked=true;
                UseRightChannel1->Checked=false;
        }

        Caption="Wave Loader - "+FileNameGetFname(FileName)+FileNameGetExt(FileName);

        UpdateImage();
        DoCaption("Stopped");
}


void __fastcall TWavLoad::LoadClick(TObject *Sender)
{
        if (LoadWavDialog->Execute()) LoadFile(LoadWavDialog->FileName);
        RecBtn->Enabled=Wav.CanRecord();
}

//---------------------------------------------------------------------------
void __fastcall TWavLoad::ScrollBarChange(TObject *Sender)
{
        if (!Playing && !Recording) TapePos=ScrollBar->Position;
        UpdateImage();
}
//---------------------------------------------------------------------------
void __fastcall TWavLoad::PlayBtnClick(TObject *Sender)
{
        if (Wav.NoSamples==0) return;

        Playing=!Playing;

        if (Playing)
        {
                PlayBtn->Caption="Stop";
                RecBtn->Enabled=false;
                DoCaption("Playing...");
        }
        else
        {
                PlayBtn->Caption="Play";
                RecBtn->Enabled = Wav.CanRecord();
                DoCaption("Stopped");
                ScrollBar->Position = TapePos;
                UpdateImage();
        }
}
//---------------------------------------------------------------------------

bool TWavLoad::GetEarState()
{
        int i;

        if (!Playing) return (0);
        
        i=128-Wav.Sample(TapePos+200,Channel);

        //if (i<0) i=-i;
        if (    (Threshold>=0 && i>=Threshold)
             || (Threshold<0 && i<Threshold) ) return(1);
        else return(0);
}

void TWavLoad::ClockTick(int TStates, bool ZX81, bool MicState)
{
        if (!Playing && !Recording) return;
        if (IgnoreZX81 && ZX81) return;

        ScreenCounter += TStates;
        CurrentTStates += TStates;

        if (CurrentTStates<TStatesSample) return;

        CurrentTStates -= TStatesSample;
        TapePos++;

        if (Recording)
        {
                static int count=0, level=128, lastbit=0;
                int curve[12] = { 36, 56, 60,  64, 64, 64,  64, 64, 64,  60, 56, 36 };

                if (MicState == lastbit) count += (zx81.machine==MACHINEACE)?1:3;
                else count=0;
                //count=0;
                if (count<12) level = MicState ? 128-curve[count] : 128+curve[count];
                else level = ((level-128)/3)+128;

                //level = MicState ? 64 : 192;

                Wav.SetSample(TapePos, level);
                lastbit=MicState;
                //ScrollBar->Max=Wav.NoSamples;
        }

        if (TapePos>Wav.NoSamples)
        {
                Stop(true);
                return;
        }

        if (ScreenCounter<1083333) return;

        ScreenCounter=0;
        ScrollBar->Max=Wav.NoSamples;
        ScrollBar->Position = TapePos;
        UpdateImage();    Application->ProcessMessages();
        if (Playing) DoCaption("Playing...");
        else DoCaption("Recording...");
}

void TWavLoad::Stop(bool force)
{
        if (!AutoStartStop1->Checked && !force) return;
        if (Playing) PlayBtnClick(NULL);
        if (Recording) RecBtnClick(NULL);
}

void TWavLoad::Start()
{
        if (!AutoStartStop1->Checked) return;
        if (Playing) return;

        if (Recording) RecBtnClick(NULL);
        PlayBtnClick(NULL);
}

void TWavLoad::StartRec()
{
        if (!AutoStartStop1->Checked) return;
        if (Recording) return;

        if (Playing) PlayBtnClick(NULL);
        RecBtnClick(NULL);
}


void TWavLoad::DoCaption(AnsiString Message)
{
        if (Message=="") Message=StatusText;
        else StatusText=Message;

        if (Wav.SampleRate>0)
        {
                Message += " ";
                Message += TapePos / Wav.SampleRate;
                Message += "s";
        }

        Caption="Wave Loader - "+FileNameGetFname(FileName)+FileNameGetExt(FileName)
                + " ("+Message+")";

}

void __fastcall TWavLoad::Image1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
        if (Shift.Contains(ssLeft))
        {
                Threshold = (64-Y)*2;
                UpdateImage();
        }

}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::OKBtnClick(TObject *Sender)
{
        Close();
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::FormClose(TObject *Sender, TCloseAction &Action)
{
        Form1->WavLoadBtn->Checked=false;
}
//---------------------------------------------------------------------------


void __fastcall TWavLoad::LeftBtnClick(TObject *Sender)
{
        Channel=0;        
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::RightBtnClick(TObject *Sender)
{
        Channel=1;
}
//---------------------------------------------------------------------------

void TWavLoad::LoadSettings(TIniFile *ini)
{
        Top = ini->ReadInteger("WAVLOAD","Top",Top);
        Left = ini->ReadInteger("WAVLOAD","Left",Left);
        Height = ini->ReadInteger("WAVLOAD","Height",Height);
        Width = ini->ReadInteger("WAVLOAD","Width",Width);

        AutoStartStop1->Checked = ini->ReadBool("WAVLOAD","AutoStart", AutoStartStop1->Checked);
        Threshold = ini->ReadInteger("WAVLOAD","Threshold",Threshold);

        LoadWavDialog->FileName = ini->ReadString("WAVLOAD","Filename",LoadWavDialog->FileName);
        LoadWavDialog->InitialDir = ini->ReadString("WAVLOAD","Dir",LoadWavDialog->InitialDir);

        if (Form1->WavLoadBtn->Checked) Show();
}

void TWavLoad::SaveSettings(TIniFile *ini)
{
        ini->WriteInteger("WAVLOAD","Top",Top);
        ini->WriteInteger("WAVLOAD","Left",Left);
        ini->WriteInteger("WAVLOAD","Height",Height);
        ini->WriteInteger("WAVLOAD","Width",Width);

        ini->WriteBool("WAVLOAD","AutoStart", AutoStartStop1->Checked);

        ini->WriteInteger("WAVLOAD","Threshold",Threshold);

        ini->WriteString("WAVLOAD","Filename",LoadWavDialog->FileName);
        ini->WriteString("WAVLOAD","Dir",LoadWavDialog->InitialDir);

}


void __fastcall TWavLoad::SaveBtnClick(TObject *Sender)
{
        AnsiString Filter = "Windows WAV Files|*.wav";

        if (FileName != "") SaveWavDialog->FileName = RemoveExt(FileName);
        else SaveWavDialog->FileName = RemoveExt(SaveWavDialog->FileName);

        SaveWavDialog->Filter = Filter;
        SaveWavDialog->DefaultExt = "wav";
        SaveWavDialog->FilterIndex=1;

        if (!SaveWavDialog->Execute()) return;

        Wav.SaveFile(SaveWavDialog->FileName);
        //Changed=false;
}
//---------------------------------------------------------------------------
AnsiString TWavLoad::RemoveExt(AnsiString Fname)
{
        AnsiString Ext;
        int len,pos;

        len=Fname.Length();

        if (len<3) return(Fname);

        pos=len;
        while(pos)
        {
                if (Fname[pos]=='.') break;
                pos--;
        }

        Ext = Fname.SubString(pos, 1+len-pos);

        if ( (Ext==".p" || Ext==".P" || Ext==".T81") && pos>1)
                Fname = Fname.SubString(1,pos-1);

        return(Fname);
}

void __fastcall TWavLoad::Send2TapeManClick(TObject *Sender)
{
	unsigned long  	i;
	BYTE	*inbuf, *dataptr;
	long	ByteCount, DataCount;
	unsigned 	buffersize = (1024 * 64);
        bool    PlayingSave,  start;
        int byte, Silence;
        BYTE  *Data;

        int StartTapePos;

        int E_LINE;

        IgnoreZX81=true;
        PlayingSave = Playing;
        //Playing=true;

        if (!Playing) PlayBtnClick(this);

	inbuf =(unsigned char *) malloc(buffersize);

        memset(inbuf, 0, buffersize);

        dataptr = inbuf;
        Data=NULL;
        ByteCount=0;
        DataCount=0;

        E_LINE=-1;
        start=true;

        StartTapePos=TapePos;

        do
        {
                byte=GetByte(start);
        }
        while((byte==-1) && Playing);

        Silence = (TapePos - StartTapePos) * TStatesSample;

        while(byte!=-1 && DataCount!=E_LINE)
        {
                if (start) start=false;
                ByteCount++;
                *(dataptr++) = byte;
                if (!Data)
                {
                        if (byte & 0x80) Data=dataptr;
                }
                else    DataCount++;

                if (DataCount==16) E_LINE= (Data[11]+Data[12]*256) - 16392;

                Application->ProcessMessages();
                byte=GetByte(start);
         }

        Tape->AddBlock(NULL, NULL, Silence);
        if (ByteCount>32) Tape->AddBlock(inbuf, Data, DataCount);

        if (!PlayingSave) PlayBtnClick(this);

        free(inbuf);
        IgnoreZX81=false;
}


int TWavLoad::GetByte(bool start)
{
	int b,e;

	int data;
	int CF,ZF;
	int ResetCounter=1;

	data=1;
	b=256;
        ClockTick(14, false, false);

	while(--b)
	{
		ClockTick(68, false, false);
                if (!Playing) return(-1);
		if (GetEarState())
		{
			e=0x94;
                        ClockTick(23, false, false);
			do
			{
				if (ResetCounter)
				{
					b=0x1a;
					ResetCounter=0;
                                        ClockTick(7, false, false);
				}
	                        if (!Playing) return(-1);

				e--;
				ClockTick(31, false, false);
				CF= GetEarState();
				ZF= ((e&128)==0);

				if (CF)
                                {
                                        ResetCounter=1;
                                        ClockTick(12, false, false);
                                }
				else
                                {
                                        b--;
                                        ClockTick(20, false, false);
                                }

			} while(CF || b);
                        ClockTick(37, false, false);
			if (ZF && (e>=0x56)) b=256;
                        else
			{
				ClockTick(8, false, false);
				data = (data<<1) | (e<32);
				if (data&256) return(data&255);
			}
		}
	}

	return(-1);
}

void __fastcall TWavLoad::NewClick(TObject *Sender)
{
        Wav.NewFile();

        FileName="";

        TStatesSample=((3250000 / Wav.SampleRate));//*7)/8;
        TapePos=0;
        CurrentTStates=0;
        Playing=0;
        Channel=0;
        ScrollBar->Max = Wav.NoSamples;
        ScrollBar->Position = 0;

        UseLeftChannel1->Enabled=false;
        UseRightChannel1->Enabled=false;
        UseLeftChannel1->Checked=false;
        UseRightChannel1->Checked=false;
        if (Wav.Stereo)
        {
                UseLeftChannel1->Enabled=true;
                UseRightChannel1->Enabled=true;
                UseLeftChannel1->Checked=true;
                UseRightChannel1->Checked=true;
        }

        Caption="Wave Loader";

        UpdateImage();
        DoCaption("Stopped");
        RecBtn->Enabled=Wav.CanRecord();
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::RecBtnClick(TObject *Sender)
{
        Recording=!Recording;

        if (Recording)
        {
                RecBtn->Caption="Stop";
                PlayBtn->Enabled=false;
                DoCaption("Recording...");
        }
        else
        {
                RecBtn->Caption="Record";
                PlayBtn->Enabled = true;
                DoCaption("Stopped");
                ScrollBar->Position = TapePos;
                UpdateImage();
        }
}
//---------------------------------------------------------------------------

void TWavLoad::EncodeBit(bool bit)
{
        int j,len;

        len = bit ? 8:4;

        while(len--)
        {
                for(j=0;j<(160);j++) ClockTick(TSClick, false, true);
                for(j=0;j<(160);j++) ClockTick(TSClick, false, false);
        }

        for(j=0;j<1280;j++) ClockTick(TSClick, false, true);
}

void TWavLoad::EncodeBlock(char *Data, int Len)
{
        int i;

        IgnoreZX81=true;

        if (!Recording) RecBtnClick(this);

        if (!Data)
        {
        }
        else
        {
                for(i=0; i<(128*TStatesSample);i++) ClockTick(TSClick,false,false);

                while(Len)
                {
                        int byte;

                        byte = *(Data++);

                        for(i=0;i<8;i++)
                        {
                                EncodeBit(byte&128);
                                byte <<= 1;
                        }

                        Len--;

                        if (Len%64) Application->ProcessMessages();
                }

                for(i=0; i<(128*TStatesSample);i++) ClockTick(TSClick,false,false);
        }

        ScrollBar->Max = Wav.NoSamples;
        ScrollBar->Position = TapePos;
        UpdateImage();

        RecBtnClick(this);
        IgnoreZX81=false;
}
void __fastcall TWavLoad::VolumeChange(TObject *Sender)
{
        Wav.Volume=-Volume->Position;
        UpdateImage(); Application->ProcessMessages();
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::BiasChange(TObject *Sender)
{
        Wav.Bias=Bias->Position;
        UpdateImage(); Application->ProcessMessages();
}
//---------------------------------------------------------------------------


void __fastcall TWavLoad::SaveWav1Click(TObject *Sender)
{
        AnsiString Filter = "Windows WAV Files|*.wav";

        if (FileName != "") SaveWavDialog->FileName = RemoveExt(FileName);
        else SaveWavDialog->FileName = RemoveExt(SaveWavDialog->FileName);

        SaveWavDialog->Filter = Filter;
        SaveWavDialog->DefaultExt = "wav";
        SaveWavDialog->FilterIndex=1;

        if (!SaveWavDialog->Execute()) return;

        Wav.SaveFile(SaveWavDialog->FileName);
        //Changed=false;
        
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::OpenWav1Click(TObject *Sender)
{
        if (LoadWavDialog->Execute()) LoadFile(LoadWavDialog->FileName);
        RecBtn->Enabled=Wav.CanRecord();
        
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::NewWav1Click(TObject *Sender)
{
        Wav.NewFile();

        FileName="";

        TStatesSample=((3250000 / Wav.SampleRate));//*7)/8;
        TapePos=0;
        CurrentTStates=0;
        Playing=0;
        Channel=0;
        ScrollBar->Max = Wav.NoSamples;
        ScrollBar->Position = 0;

        UseLeftChannel1->Enabled=false;
        UseRightChannel1->Enabled=false;
        UseLeftChannel1->Checked=false;
        UseRightChannel1->Checked=false;
        if (Wav.Stereo)
        {
                UseLeftChannel1->Enabled=true;
                UseRightChannel1->Enabled=true;
                UseLeftChannel1->Checked=true;
                UseRightChannel1->Checked=false;
        }

        Caption="Wave Loader";

        UpdateImage();
        DoCaption("Stopped");
        RecBtn->Enabled=Wav.CanRecord();
        
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::ConvertNextBlock1Click(TObject *Sender)
{
	unsigned long  	i;
	BYTE	*inbuf, *dataptr;
	long	ByteCount, DataCount;
	unsigned 	buffersize = (1024 * 64);
        bool    PlayingSave,  start;
        int byte, Silence;
        BYTE  *Data;

        int StartTapePos;

        int E_LINE;

        IgnoreZX81=true;
        PlayingSave = Playing;
        //Playing=true;

        if (!Playing) PlayBtnClick(this);

	inbuf =(unsigned char *) malloc(buffersize);

        memset(inbuf, 0, buffersize);

        dataptr = inbuf;
        Data=NULL;
        ByteCount=0;
        DataCount=0;

        E_LINE=-1;
        start=true;

        StartTapePos=TapePos;

        do
        {
                byte=GetByte(start);
        }
        while((byte==-1) && Playing);

        Silence = (TapePos - StartTapePos) * TStatesSample;

        while(byte!=-1 && DataCount!=E_LINE)
        {
                if (start) start=false;
                ByteCount++;
                *(dataptr++) = byte;
                if (!Data)
                {
                        if (byte & 0x80) Data=dataptr;
                }
                else    DataCount++;

                if (DataCount==16) E_LINE= (Data[11]+Data[12]*256) - 16392;

                Application->ProcessMessages();
                byte=GetByte(start);
        }

        if (Sender)
        {
                Tape->AddBlock(NULL, NULL, Silence);
                if (ByteCount>32) Tape->AddBlock(inbuf, Data, DataCount);
        }
        else
        {
                TZX->AddBlock(NULL,Silence);
                if (ByteCount>32) TZX->AddBlock(inbuf, ByteCount);
                TZX->MergeBlocks();
                TZX->UpdateTable();
        }

        if (!PlayingSave) PlayBtnClick(this);

        free(inbuf);
        IgnoreZX81=false;

}
//---------------------------------------------------------------------------


void __fastcall TWavLoad::UseLeftChannel1Click(TObject *Sender)
{
        UseLeftChannel1->Checked=true;
        UseRightChannel1->Checked=false;
        Channel=0;
}
//---------------------------------------------------------------------------


void __fastcall TWavLoad::UseRightChannel1Click(TObject *Sender)
{
        UseLeftChannel1->Checked=false;
        UseRightChannel1->Checked=true;
        Channel=1;
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::AutoStartStop1Click(TObject *Sender)
{
        AutoStartStop1->Checked = !AutoStartStop1->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TWavLoad::ConvertNextBlockTZX1Click(TObject *Sender)
{
        ConvertNextBlock1Click(NULL);
        
}
//---------------------------------------------------------------------------

