#include "z80.h"
#include "zx81.h"
#include "accurate.h"
#include "snap.h"
#include "zx81config.h"
#include "WavCInterface.h"
#include "zxprinter_c.h"
#include "rompatch.h"
#include "tzxman.h"

#define DEBUG 1

#define HBLANKCOLOUR 0
#define VBLANKCOLOUR 0

#define BLACK 0
#define WHITE 246
#define INV_BLACK 0
#define INV_WHITE 255

extern void debug(int data);
extern void DebugUpdate(void);
extern int RasterY, LOOPER=207;

int border=7, ink=0, paper=7;

int NMI_generator=0;
int HSYNC_generator=0;
int sync_len=0, sync_valid=0;
int rowcounter=0;
int hsync_counter=207;
int borrow=0;
int ACEMICState;

int int_pending=0;

int shift_register=0, shift_reg_inv, shift_store=0;
long noise;
BYTE scanline[800*50];
int scanline_len=0;
BYTE scandata[32];

void zx81_accurate_init(void)
{
        NMI_generator=0;
        HSYNC_generator=0;
        sync_len=0;
        sync_valid=0;

        LOOPER=207;
        // NASTY HACK ALERT!

        if (zx81.romcrc==CRCH4TH || zx81.romcrc==CRCTREE4TH)
                LOOPER=206;

        if (zx81.romcrc==CRCASZMIC || zx81.romcrc==CRC8300 || zx81.romcrc==CRCLAMBDA)
                 LOOPER=208;
        z80_reset();
}

#if DEBUG==1
void add_blank(int tstates, BYTE colour)
{
        while(tstates-- > 0)
        {
                scanline[scanline_len++]=colour;
                scanline[scanline_len++]=colour;
        }
}
#else
#define add_blank(a,b)
#endif

int zx81_do_accurate()
{
        int ts,i;
        int MaxScanLen;
        int PrevRev=0, PrevBit=0;
        int tstotal=0;

        static int LambdaNMI=0;

        scanline_len=0;

        MaxScanLen = zx81.single_step? 1:420;

        if (sync_valid)
        {
                add_blank(-borrow, HSYNC_generator ?
                        (128+4*(zx81.inverse?ink:paper)) : VBLANKCOLOUR );
                sync_valid=0;
        }

        do
        {
                z80.pc.w=PatchTest(z80.pc.w);

                ts = z80_do_opcode();

                if (int_pending)
                {
                        int intlen;
                        intlen = z80_interrupt(ts);
                        paper=border;
                        ts += intlen;
                        int_pending=0;
                }

                frametstates += ts;
                WavClockTick(ts, HSYNC_generator);
                ZXPrinterClockTick(ts);
                TZXClockTick(ts);

                shift_store=shift_register;
                for (i=0; i<(ts<<1); i++)
                {
                        int colour, bit;

                        bit=((shift_register^shift_reg_inv)&32768);
                        if (zx81.inverse) bit=!bit;

                        colour = (bit ? ink:paper)*4;

                        if (PrevBit && PrevRev && zx81.dirtydisplay)
                                colour+=2;
                        if ((noise&1) && zx81.dirtydisplay)
                                colour+=1;

                        scanline[scanline_len++] =
                                HSYNC_generator ?  colour+128 :VBLANKCOLOUR;

                        PrevRev=shift_reg_inv&32768;
                        PrevBit= bit;

                        shift_register<<=1;
                        shift_reg_inv<<=1;
                        noise>>=1;
                }

                hsync_counter -= ts;

                if (!(z80.r & 64)) int_pending=1;
                if (!HSYNC_generator) sync_len += ts;


                if (hsync_counter<=0)
                {
                        if (NMI_generator)
                        {
                                int nmilen;
                                nmilen = z80_nmi(scanline_len);
                                if (nmilen!=11) add_blank(nmilen-11,60);
                                hsync_counter -= nmilen;
                                ts += nmilen;
                                LambdaNMI=1;
                        }

                        borrow=hsync_counter;
                        if (HSYNC_generator && sync_len==0)
                        {
                                sync_len=16;
                                sync_valid=1;
                                if (scanline_len>=(LOOPER*2))
                                {
                                        int i;
                                        scanline_len=LOOPER*2;

                                }
                                for(i=0;i<24;i++) scanline[i]=HBLANKCOLOUR;

                                //add_blank(16,HBLANKCOLOUR);
                                rowcounter = (++rowcounter)&7;
                        }
                        hsync_counter += LOOPER;
                }

                tstotal += ts;

                DebugUpdate();

        } while(scanline_len<MaxScanLen && !sync_valid && !zx81_stop);

        if (sync_valid && sync_len>64)
        {
                hsync_counter=LOOPER;
                borrow=0;
        }

        return(tstotal);
}

void debug(int data)
{
        add_blank(2,data);
}

int ace_do_accurate()
{
        int ts,i,j;
        int topborder=56;
        static int Sy=0, loop=208;
        scanline_len=0;

        //if (i==208) Scanline_len=0;
        if (zx81.NTSC) topborder=32;

        if (Sy>304)
        {
                for(i=0;i<414;i++)
                {
                        scanline[scanline_len++] = 0;
                }
                sync_len=414;
                sync_valid=1;
                Sy=0;
                z80_interrupt(ts);
                ts=208;
        }
        else
        {
                while(loop>0 && !zx81_stop)
                {
                        z80.pc.w=PatchTest(z80.pc.w);
                        ts=z80_do_opcode();
                        loop-=ts;
                        frametstates += ts;
                        WavClockTick(ts, ACEMICState);
                        ZXPrinterClockTick(ts);

                        for(i=0;i<ts;i++)
                        {
                                scanline[scanline_len++]=noise&1;
                                scanline[scanline_len++]=noise&1;
                                noise>>=1;
                        }

                        DebugUpdate();
                }
                scanline_len=0;

                ts=loop;
                if (loop<=0)
                {
                        loop=208+loop;
                        if (Sy<topborder || Sy>topborder+191)
                                for(i=0;i<414;i++)
                                {
                                        scanline[scanline_len]+=128+(zx81.inverse?border:7-border)*4;
                                        scanline_len++;
                                }

                        else
                        {
                                for(i=0;i<68;i++)
                                {
                                        scanline[scanline_len]+=128+(zx81.inverse?border:7-border)*4;
                                        scanline_len++;
                                }
                                for(i=0;i<32;i++)
                                {
                                        int inv,bitmap,chr, attr;
                                        chr=memory[0x2400+i+((Sy-topborder)/8)*32];
                                        attr=acecolour[i+((Sy-topborder)/8)*32];

                                        ink=attr&7;
                                        paper=(attr>>4)&7;

                                        inv=chr&128; chr=chr&127;
                                        bitmap=memory[0x2c00+chr*8+(Sy-topborder)%8];
                                        if (inv) bitmap = (~bitmap)&255;

                                        for(j=0;j<8;j++)
                                        {
                                                scanline[scanline_len]+=128+((bitmap&128)?
                                                        (zx81.inverse?(7-ink):ink):zx81.inverse?(7-paper):paper)*4;
                                                scanline_len++;
                                                bitmap=(bitmap<<1)&255;
                                        }
                                }

                                for(i=0;i<78;i++)
                                {
                                        scanline[scanline_len]+=128+(zx81.inverse?border:7-border)*4;
                                        scanline_len++;
                                }
                        }
                        sync_len=16;
                        sync_valid=1;
                        for(i=0;i<16;i++)
                        {
                                scanline[scanline_len++] = HBLANKCOLOUR;
                        }
                        Sy++;
                }
        }

        return(208-ts);
}
