#include <stdlib.h>
#include <fcntl.h>
#include <io.h>

#include "zx81.h"
#include "z80\z80.h"
#include "snap.h"
#include "accurate.h"
#include "zx81config.h"
#include "accurate.h"
#include "WavCInterface.h"
#include "sound.h"
#include "dev8255.h"
#include "serialport.h"

#define TZX_BYTE_EMPTY -1

extern void font_load(char *filename, void *address, int size);
extern void ZXPrinterWritePort(unsigned char Data);
extern unsigned char ZXPrinterReadPort(void);
extern int hsync_counter, rowcounter;
extern int shift_register, shift_reg_inv;
extern long noise;
extern int ACEMICState;
extern int ink, paper, border;

extern int hsync_counter;
extern int TZXByte, TZXEventCounter;

extern void TZXWriteByte(unsigned char Byte);

int tstates, frametstates;
int event_next_event;
int zx81_stop=0;
BYTE memory[65536];
BYTE font[512];
BYTE memhrg[1024];
BYTE acecolour[1024], acelatch=4;

int kbBreak=0;

static int ayreg;


//int memhrgenable=0;

BYTE ZXKeyboard[8];

void set_PC(int addr)
{
        z80.pc.w=addr;
}

BYTE get_i_reg(void)
{
        //return(0x1e);
        return(z80.i);
}

void zx81_init(void)
{
        int i, romlen;
        z80_init();

        for(i=0;i<65536;i++) memory[i]=7;

        switch (zx81.romtype)
        {
        case 0: romlen=memory_load(zx81.ROM80, 0, 65536); break;
        case 1: romlen=memory_load(zx81.ROM81, 0, 65536); break;
        case 2: romlen=memory_load(zx81.ROM97LE, 0, 65536); break;
        case 3: romlen=memory_load(zx81.ROMTS1000, 0, 65536); break;
        case 4: romlen=memory_load(zx81.ROMTS1500, 0, 65536); break;
        case 5: romlen=memory_load(zx81.ROMLAMBDA, 0, 65536); break;
        case 6: romlen=memory_load(zx81.ROMPC8300, 0, 65536); break;
        case 7: romlen=memory_load(zx81.ROMR470, 0, 65536); break;
        case 8: romlen=memory_load(zx81.ROMTK85, 0, 65536); break;
        case 9: romlen=memory_load(zx81.ROMACE, 0, 65536); break;
        }

        if (zx81.romcrc==-1)
                zx81.romcrc=CRC32Block(memory,romlen);
        if (zx81.extfont) font_load("lmbfnt.rom",font,512);

        if (zx81.chrgen==CHRGENDK) romlen+=memory_load("dkchr.rom",8192,65536);

        if (zx81.shadowROM && romlen<=8192)
        {
                for(i=0;i<8192;i++) memory[i+8192]=memory[i];
                zx81.ROMTOP=16383;
        }
        else    zx81.ROMTOP=romlen-1;

        if (zx81.machine==MACHINEZX97LE)
        {
                for(i=0;i<8191;i++) memory[i+0xa000]=memory[i+0x2000];
                for(i=0;i<16384;i++) zx97.bankmem[i]=memory[i+0x4000];
                //for(i=8192;i<32768;i++) memory[i]=0x07;
                zx81.ROMTOP=8191;
        }

        acelatch= (zx81.colour==COLOURACE) ? 4:7;


        if (zx81.truehires==HIRESMEMOTECH) memory_load("memohrg.rom", 8192, 2048);
        if (zx81.truehires==HIRESG007) memory_load("g007hrg.rom",10240,2048);

        ink=0; paper=border=7;

        zx81_accurate_init();
        d8255_reset();
        d8251reset();
        z80_reset();
}

void writebyte(int Address, int Data)
{
        noise = (noise<<8) | Data;

        if (zx81.aytype == AY_TYPE_QUICKSILVA)
        {
                if (Address == 0x7fff) ayreg=Data&15;
                if (Address == 0x7ffe) sound_ay_write(ayreg,Data);
        }

        if (zx81.machine == MACHINEACE)
        {
                if (Address<8192) return;

                if (Address==0x2700 && (Data&128)) acelatch=Data&127;

                if (Address>=0x2000 && Address<=0x23ff) Address += 0x400;
                if (Address>=0x2800 && Address<=0x2bff) Address += 0x400;
                if (Address>=0x3000 && Address<=0x33ff) Address += 0xc00;
                if (Address>=0x3400 && Address<=0x37ff) Address += 0x800;
                if (Address>=0x3800 && Address<=0x3bff) Address += 0x400;

                if (Address>=0x2400 && Address<=0x27ff) acecolour[Address-0x2400]=acelatch;

                if ( (Address<=zx81.ROMTOP && zx81.protectROM)
                        || Address>zx81.RAMTOP) return;

                noise =0;
                memory[Address]=Data;
                return;
        }


        if (zx81.colour==COLOURLAMBDA && (Address>=8192 && Address<16384))
        {
                Address = (Address&1023)+8192;
                memory[Address]=Data;
                return;
        }

        if (zx81.machine==MACHINEZX97LE)
        {
                //if (Address>0x9fff && zx81.protectROM) return;
                //if (Address<0x2000) return;

                if (zx97.protect08 && Address<0x2000) return;
                if (zx97.protectab && Address>=0xa000 && Address<0xc000) return;

                if (Address>=49152)
                {
                        if (!(d8255_read(D8255PRTB)&16)) return;
                        if (zx97.protectb0 && ((d8255_read(D8255PRTB)&15)==0)) return;
                        if (zx97.protectb115 && ((d8255_read(D8255PRTB)&15)>0)) return;

                        zx97.bankmem[(Address&16383) +  16384*(d8255_read(D8255PRTB)&15)]=Data;
                        return;
                }

                if (zx97.bankswitch && Address>0x7fff && Address<=0x9fff) Address -=8000;
                if (zx97.bankswitch && Address<8192) Address += 0x8000;
        }

        //if (Address>=32768 && zx81.shadowRAM) Address = Address &32767;

        if (Address>=0x8400 && Address<=0x87ff && zx81.chrgen==CHRGENQS)
        {
                font[Address-0x8400]=Data;
                zx81.enableqschrgen=1;
        }

        if (Address>zx81.RAMTOP) Address = (Address&(zx81.RAMTOP));//-16384))+16384;

        if (Address<=zx81.ROMTOP && zx81.protectROM)
        {
                if (zx81.truehires==HIRESMEMOTECH && Address<1024)
                                memhrg[Address]=Data;
                return;
        }

        if (Address>8191 && Address<16384 && zx81.shadowROM && zx81.protectROM) return;
        if (Address<10240 && zx81.truehires==HIRESMEMOTECH) return;
        if (Address>=10240 && Address<12288 && zx81.truehires==HIRESG007) return;

        if (Address<=zx81.RAMTOP) memory[Address]=Data;
}

BYTE readbyte(int Address)
{
        int data;

        if (zx81.machine == MACHINEACE)
        {
                if (Address>=0x2000 && Address<=0x23ff) Address += 0x400;
                if (Address>=0x2800 && Address<=0x2bff) Address += 0x400;
                if (Address>=0x3000 && Address<=0x33ff) Address += 0xc00;
                if (Address>=0x3400 && Address<=0x37ff) Address += 0x800;
                if (Address>=0x3800 && Address<=0x3bff) Address += 0x400;

                if (Address>=0x2800 && Address<=0x2fff) return(255);

                data=memory[Address];
                //noise = (noise<<8) | data;
                return(data);
        }

        if (zx81.colour==COLOURLAMBDA && ((Address>=8192 && Address<16384)
                                          || (Address>=49152 && Address<57344)))
        {
                Address = (Address&1023)+8192;
                data=memory[Address];
                return(data);
        }

        if (zx81.machine==MACHINEZX97LE)
        {
                if (zx97.bankswitch && Address<8192) Address+=0x8000;
                else if (zx97.bankswitch && (Address>=0x8000 && Address<=0x9fff)) Address-=0x8000;

                if (Address>=49152)
                {
                        data=zx97.bankmem[(Address&16383) + (d8255_read(D8255PRTB)&15)*16384];
                        noise = (noise<<8) | data;
                        return(data);
                }
        }

        if (Address<=zx81.RAMTOP) data=memory[Address];
        else data=memory[(Address&(zx81.RAMTOP-16384))+16384];

        if ((Address<1024 && (zx81.truehires==HIRESMEMOTECH)) && (z80.i&1))
                        data=memhrg[Address];

        if ((Address>=0x0c00 && Address<=0x0cff) && (zx81.truehires==HIRESG007))
                data=memory[Address+8192];

        if ((Address<256 || (Address>=512 && Address<768))
                && (z80.i&1) && (zx81.truehires==HIRESG007))
                        data=memory[Address+8192];

        noise = (noise<<8) | data;
        return(data);
}

BYTE opcode_fetch(int Address)
{
        int NewAddress, inv;
        int opcode, bit6, update=0;
        BYTE data;
        static int setborder=0;

        if (zx81.machine == MACHINEACE) return(memory[Address]);

        if (Address==0) zx81.enableqschrgen=0;

        data = readbyte((Address>=zx81.m1not)?Address&32767:Address);
        opcode=data;
        bit6=opcode&64;

        if (Address>=zx81.m1not && !bit6) opcode=0;

        if (Address>=zx81.m1not)
        {
                int maxireg;

                inv = data&128;

                maxireg=32;
                if (zx81.chrgen==CHRGENCHR16 && z80.i<128) maxireg=64;
                if (zx81.chrgen==CHRGENDK) maxireg=64;


                if (z80.i >=maxireg && zx81.truehires==HIRESWRX && !bit6)
                {
                        data=readbyte((z80.i<<8) | (z80.r7 & 128) | ((z80.r-1) & 127));
                        update=1;
                }
                else
                if (z80.i&1 && zx81.truehires==HIRESMEMOTECH)
                {
                        static int HaltCount=0,LineLen=-1;

                        opcode=0;
                        inv=0;
                        update=1;
                        LineLen++;

                        if (data&63)
                        {
                                if (HaltCount==0 && LineLen==0)
                                {
                                        opcode=data;
                                        update=0;
                                        HaltCount++;
                                        LineLen=-1;
                                }
                                else
                                if (LineLen>31)
                                {
                                        LineLen=-1;
                                        opcode=data;
                                        update=0;
                                        HaltCount++;
                                        if (HaltCount==193) HaltCount=0;
                                }
                        }
                }
                else
                if (z80.i&1 && zx81.truehires==HIRESG007)
                {
                        opcode=0;
                        inv=0;
                        update=1;
                }
                else
                if (!bit6)
                {
                        if ((zx81.chrgen==CHRGENCHR16 && (z80.i&1)) || (zx81.chrgen==CHRGENQS && zx81.enableqschrgen))
                                data = ((data&128)>>1)|(data&63);
                        else    data = data&63;

                        if (z80.i<64 || (z80.i>=128 && z80.i<192 && zx81.chrgen==CHRGENCHR16))
                        {
                                if (zx81.extfont || (zx81.chrgen==CHRGENQS && zx81.enableqschrgen))
                                        data= font[(data<<3) | rowcounter];
                                else    data=readbyte(((z80.i&254)<<8) + (data<<3) | rowcounter);
                        }
                        else data=255;

                        update=1;
                }

                if (update)
                {
                        if (zx81.colour==COLOURLAMBDA)
                        {
                                int i,p,c;

                                c=readbyte((Address&1023)+8192);

                                ink = (~c)&7;
                                paper = ((~c)>>4) & 7;
                                if (setborder)
                                {
                                        border=paper;
                                        setborder=0;
                                }
                        }


                        shift_register |= data;
                        shift_reg_inv |= inv? 255:0;
                        noise |= z80.i;
                        return(0);
                }

        }

        noise |= data;
        setborder=1;
        return(opcode);
}

void writeport(int Address, int Data)
{
        static int beeper=0;
        unsigned char *p;

        /* the examples in the manual (using DF/0F) and the
         * documented ports (CF/0F) don't match, so decode is
         * important for that.
         */
        //if(!(Address&0xf0)) Address=0x0f;
        //else if (!(Address&0x20)) Address=0xdf;

        switch(Address&255)
        {
        case 0x0f:
                if (zx81.aytype==AY_TYPE_ZONX)
                        sound_ay_write(ayreg, Data);
                break;

        case 0xdf:
                if (zx81.aytype==AY_TYPE_ACE) sound_ay_write(ayreg, Data);
                if (zx81.aytype==AY_TYPE_ZONX) ayreg=Data&15;
                break;
        case 0x3f:
                if (zx81.aytype==AY_TYPE_FULLER)
                        ayreg=Data&15;
        case 0x5f:
                if (zx81.aytype==AY_TYPE_FULLER)
                        sound_ay_write(ayreg, Data);
                break;

        case 0x73:
                if (zx81.ts2050) d8251writeDATA(Data);
                break;
        case 0x77:
                if (zx81.ts2050) d8251writeCTRL(Data);
                break;

        case 0xc7:
                d8255_write(D8255PRTA,Data);
                break;

        case 0xcf:
                d8255_write(D8255PRTB,Data);
                break;

        case 0xd7:
                d8255_write(D8255PRTC,Data);
                break;

        case 0xdd:
                if (zx81.aytype==AY_TYPE_ACE) ayreg=Data;
                break;

        case 0xfb:
                ZXPrinterWritePort(Data);
                break;

        case 0xfd:
                if (zx81.machine==MACHINEZX80) break;
                NMI_generator=0;
                tstates-=4;
                //tstates-=8;
                break;
        case 0xfe:
                if (zx81.machine==MACHINEZX80) break;
                ACEMICState = Data&8;
                NMI_generator=1;
                if (zx81.machine == MACHINEACE)
                {
                        beeper = 1-beeper;
                        sound_beeper(beeper);
                }
                break;
        default:
                break;
        }

        if (zx81.machine != MACHINEACE) sound_beeper(1);
        if (!HSYNC_generator) rowcounter=0;
        if (sync_len) sync_valid=1;
        HSYNC_generator=1;
        //hsync_counter=207;

        p=memory+z80.pc.w-4;

        if (zx81.TZXin && p[0]==0xdb && p[1]==0xfe && p[2]==0xd3 && p[3]==0xff)
        {
                TZXStartPlaying();
                if (TZXByte!=TZX_BYTE_EMPTY)
                {
                        z80.bc.b.l=TZXByte;
                        z80.pc.b.l=memory[z80.sp.w++];
                        z80.pc.b.h=memory[z80.sp.w++];
                        TZXByte=TZX_BYTE_EMPTY;
                        TZXEventCounter=0;
                }
        }

        if (zx81.TZXout && p[1]==0x4f && p[2]==0xd3 && p[3]==0xff && p[4]==0x06)
        {
                TZXWriteByte(memory[z80.hl.w]);
                z80.pc.b.l=memory[z80.sp.w++];
                z80.pc.b.h=memory[z80.sp.w++];
        }
}

BYTE readport(int Address)
{

        if (!(Address&1))
        {
                BYTE keyb, data=0;
                int i;
                if (zx81.machine != MACHINEACE) sound_beeper(0);

                if (zx81.NTSC) data|=64;
                switch (zx81.machine)
                {
                case MACHINEACE:
                        if (!GetEarState()) data |= 32;
                default:
                        if (zx81.TZXin)
                        {
                                if (!TZXGetEarState()) data |= 128;
                        }
                        else
                        {
                                if (!GetEarState()) data |= 128;
                        }
                        break;
                }

                if (!NMI_generator)
                {
                        HSYNC_generator=0;
                        if (sync_len==0) sync_valid=0;
                }

                keyb=Address/256;
                for(i=0; i<8; i++)
                {
                        if (! (keyb & (1<<i)) ) data |= ZXKeyboard[i];
                }

                if (kbBreak )//&& (keyb & (1<<kbA15)))
                {
                        kbBreak--;
                        data |= 1;
                }

                return(~data);
        }
        else
                switch(Address&255)
                {
                case 0x73:
                        if (zx81.ts2050) return(d8251readDATA());

                case 0x77:
                        if (zx81.ts2050) return(d8251readCTRL());

                case 0xdd:
                        if (zx81.aytype==AY_TYPE_ACE)
                                return(sound_ay_read(ayreg));
                case 0xfb:
                        return(ZXPrinterReadPort());
                default:
                        break;
                }
        return(255);
}
void ramwobble(int now)
{
        int start, length, data;
        int i;

        start=zx81.ROMTOP+1;
        length=zx81.RAMTOP-start;
        data=random(256);

        if (now || !random(64))
                for(i=0;i<length;i++) memory[start+i] ^= data;
}
        /*void ramwobble(void)
{
        int start, length, data, addr;
        int row, col;
        int type,i;

        start=zx81.ROMTOP+1;
        length=zx81.RAMTOP-start;
        addr=random(length);
        data=random(256);
        if (random(2)) data |= 64;

        row=addr&127;
        col=(addr>>7)&127;


        switch(3)//random(64))
        {
        case 0:
                memory[addr+start]=data;
                break;

        case 1:
                for(i=0;i<128;i++) memory[start+(col<<7)+i]=data;
                break;

        case 2:
                for(i=0;i<128;i++) memory[start+(i<<7)+row]=data;
                break;

        case 3:
                for(i=0;i<length;i++) memory[start+i] ^= data;

        default:
                break;
        }
}
*/
