//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "zxprinter.h"
#include "PrintOptions_.h"
#include "main_.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TPrinter *Printer;
//---------------------------------------------------------------------------
void TPrinter::ClearBitmap(Graphics::TBitmap *Bitmap)
{
        int x,y;

        Bitmap->Canvas->Pen->Color = Bg;
        for(y=0; y < Bitmap->Height; y++)
        {
                Bitmap->Canvas->MoveTo(0,y);
                Bitmap->Canvas->LineTo(Bitmap->Width,y);
        }
}
//---------------------------------------------------------------------------
void TPrinter::ResetPrinter(void)
{
        MotorOn=false;
        if (XPos) OutputLine();
        XPos=0;
        Counter2=16;
        StylusActive=false;
}
void TPrinter::OutputBit(void)
{
        if (XPos<1 || XPos>(PrinterOutput->Width)) return;
        PrinterOutput->Canvas->Pixels[XPos-1][YPos]= StylusActive?Fg:Bg;
}

void TPrinter::OutputLine(void)
{
        YPos++;
        if (YPos>=PrinterOutput->Height-1) ScrollOutput();
        DrawImage();
}

void TPrinter::ScrollOutput()
{
        int y;
        Graphics::TBitmap *Bitmap;

        Bitmap=PrinterOutput;

        PrinterOutput->Height += 8;
        ScrollBar->Max = PrinterOutput->Height;
        ScrollBar->Position += 8;

        Bitmap->Canvas->Pen->Color = Bg;
        for(y=Bitmap->Height-8; y < Bitmap->Height; y++)
        {
                Bitmap->Canvas->MoveTo(0,y);
                Bitmap->Canvas->LineTo(Bitmap->Width,y);
        }
}

//---------------------------------------------------------------------------
__fastcall TPrinter::TPrinter(TComponent* Owner)
        : TForm(Owner)
{
        TIniFile *ini;

        Fg=clBlack;
        Bg=clWhite;

        PrinterOutput= new Graphics::TBitmap;
        PrinterOutput->PixelFormat=pf4bit;
        PrinterOutput->Width = 256;
        PrinterOutput->Height = 8;

        ClearBitmap(PrinterOutput);
        DrawImage();

        MotorOn=StylusActive=OnPaper=false;
        XPos=0;
        YPos=0;//PrinterOutput->Height-8;
        Counter1=25;
        Counter2=16;

        ScrollBar->Position = PrinterOutput->Height;
        ScrollBar->Min=0;
        ScrollBar->Max=PrinterOutput->Height;

        ini = new TIniFile(ChangeFileExt(Application->ExeName, ".INI" ));
        LoadSettings(ini);
        delete ini;
}
//---------------------------------------------------------------------------
void __fastcall TPrinter::DrawImage()
{
        TRect source,dest;
        int pos;

        pos=ScrollBar->Position;

        if (pos<DisplayImage->Height) pos=DisplayImage->Height;

        source.Left=0;
        source.Top=pos - DisplayImage->Height;
        source.Right=PrinterOutput->Width;
        source.Bottom=pos;

        if (source.Top < 0) source.Top=0;

        dest.Left=0;
        dest.Top=0;
        dest.Right = DisplayImage->Width;
        dest.Bottom = DisplayImage->Height;

        DisplayImage->Canvas->CopyRect(dest, PrinterOutput->Canvas, source);
}
//---------------------------------------------------------------------------
void TPrinter::ClockTick(int ts)
{
        if (!MotorOn) return;
        Counter1-=ts;
        if (Counter1>0) return;
        Counter1 += 1016 - ((100-PrintOptions->BitSpeed) * 10);

        if (Counter2)
        {
                OnPaper=false;
                EncoderWheel=false;
                Counter2--;
        }
        else
        {
                OnPaper=true;
                EncoderWheel=true;
                OutputBit();
                XPos++;

                if (XPos==258)
                {
                        OutputLine();
                        OnPaper=false;
                        EncoderWheel=false;
                        XPos=0;
                        Counter2=1+PrintOptions->LineSpeed*10;
                }
        }
}
void TPrinter::WritePort(unsigned char Data)
{
        if (Data&4) ResetPrinter();
        else MotorOn=true;

        if (Data&128) StylusActive=true;
        else StylusActive=false;

        OnPaper=false;
        EncoderWheel=false;
}

unsigned char TPrinter::ReadPort(void)
{
        unsigned char Data=0;

        if (OnPaper) Data=128;
        //else Data=0;

        if (EncoderWheel) Data |= 1;

        return(Data);

}


//---------------------------------------------------------------------------
extern "C" void ZXPrinterClockTick(int ts)
{
        Printer->ClockTick(ts);
}
extern "C" void ZXPrinterWritePort(unsigned char Data)
{
        Printer->WritePort(Data);
}

extern "C" unsigned char ZXPrinterReadPort(void)
{
        return(Printer->ReadPort());
}


void __fastcall TPrinter::ScrollBarChange(TObject *Sender)
{
        DrawImage();
}
//---------------------------------------------------------------------------

void __fastcall TPrinter::ClearImageClick(TObject *Sender)
{
        PrinterOutput->Width = 256;
        PrinterOutput->Height = 8;

        ClearBitmap(DisplayImage->Picture->Bitmap);
        ClearBitmap(PrinterOutput);
        DrawImage();

        //MotorOn=StylusActive=OnPaper=false;
        XPos=0;
        YPos=0;
        Counter2=16;

        ScrollBar->Min=1;
        ScrollBar->Max=PrinterOutput->Height;
        ScrollBar->Position = PrinterOutput->Height;
}
//---------------------------------------------------------------------------

void __fastcall TPrinter::SaveImageClick(TObject *Sender)
{
        if (SaveDialog->Execute())
                PrinterOutput->SaveToFile(SaveDialog->FileName);
}
//---------------------------------------------------------------------------

void __fastcall TPrinter::FormClose(TObject *Sender, TCloseAction &Action)
{
        PrintOptions->Close();
        Form1->ViewPrinter->Checked=false;
}
//---------------------------------------------------------------------------


void __fastcall TPrinter::OptionsBtnClick(TObject *Sender)
{
        PrintOptions->Show();
}
//---------------------------------------------------------------------------

void TPrinter::LoadSettings(TIniFile *ini)
{
        Top = ini->ReadInteger("PRINTER","Top",Top);
        Left = ini->ReadInteger("PRINTER","Left",Left);

        PrintOptions->Top = ini->ReadInteger("PRINTER","OptTop",PrintOptions->Top);
        PrintOptions->Left = ini->ReadInteger("PRINTER","OptLeft",PrintOptions->Left);

        PrintOptions->BitSpeed = ini->ReadInteger("PRINTER","BitSpeed",PrintOptions->BitSpeed );
        PrintOptions->LineSpeed = ini->ReadInteger("PRINTER","LineSpeed",PrintOptions->LineSpeed );
        PrintOptions->BitRate->Position = ini->ReadInteger("PRINTER","SBitSpeed",PrintOptions->BitRate->Position );
        PrintOptions->LineRate->Position = ini->ReadInteger("PRINTER","SLineSpeed",PrintOptions->LineRate->Position );
        PrintOptions->Style->ItemIndex = ini->ReadInteger("Printer","Style",PrintOptions->Style->ItemIndex );

        PrintOptions->BitRateChange(NULL);
        PrintOptions->LineRateChange(NULL);
        PrintOptions->StyleChange(NULL);

        ClearImageClick(NULL);

        if (Form1->ViewPrinter->Checked) Show();
}

void TPrinter::SaveSettings(TIniFile *ini)
{
        ini->WriteInteger("PRINTER","Top",Top);
        ini->WriteInteger("PRINTER","Left",Left);

        ini->WriteInteger("PRINTER","OptTop",PrintOptions->Top);
        ini->WriteInteger("PRINTER","OptLeft",PrintOptions->Left);

        ini->WriteInteger("PRINTER","BitSpeed",PrintOptions->BitSpeed );
        ini->WriteInteger("PRINTER","LineSpeed",PrintOptions->LineSpeed );
        ini->WriteInteger("PRINTER","SBitSpeed",PrintOptions->BitRate->Position );
        ini->WriteInteger("PRINTER","SLineSpeed",PrintOptions->LineRate->Position );
        ini->WriteInteger("Printer","Style",PrintOptions->Style->ItemIndex );

}
